/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: BST_Insert.java,v 1.23 2004/05/26 08:05:40 vkaravir Exp $ */

package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.CDT.probe.BinSearchTree;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.Key;
import matrix.util.Note;


public class BST_Insert implements SimulationExerciseModel, ModelAnswerNames,
                                   ConfigureVisualType {
    Table t;
    ExerBin bt;
    BinSearchTree bst;
    public static final boolean DEBUG = false;
    //Head h;
    // model solution animator marker
    String marker = "BST_Insert";
    private String S = null;

    public BST_Insert() {
    }


    long seed = 1;;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }



    public FDT[] init() {

        BinSearchTree testBST;
        Random R = new Random(seed);
        do {
            // 15-20 keys
            int n = Math.abs(R.nextInt()%6);
            // Can contain duplicates
            S = matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,15)+
                matrix.util.RandomKey.createNoDuplicateUppercaseRandomKey(R,n);
            // Maximum height 6
            testBST = new BinSearchTree();
            for(int i = 0; i < S.length(); i++)
                testBST.insert(new Key(""+S.charAt(i)));
            if (DEBUG)
                System.out.println("iteration - height "+testBST.getHeight());
        } while(testBST.getHeight() > 6);

        t = new Table(S);
        bt = new ExerBin();
        FDT[]  s = { t, bt };
        return s;
    }

    public String[] getStructureNames() {
        String[] s = { "Stream of Keys", "Binary Search Tree" };
        return s;
    }

    public String getDescription() {
        return "";
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public String[] getModelAnswerNames() {
        String[] s = { "Binary Search Tree" };
        return s;
   }


    public FDT[] solve() {
        Note.out(this, "Solve");
        bst = new BinSearchTree();
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for (int i=0; i<tbl.size(); i++) {
            a.startOperation();
            bst.insert(tbl.getObject(i));
            a.endOperation();
        }
        Note.out(this, "Solved");
        return new FDT[] { (FDT)bst.getElement() };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public FDT[] getAnswer() {
        FDT[] s = { bt };
        return s;
    }

    public FDT[] getInitialStructures() {
        return new FDT[] { new Table(S), new ExerBin() };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, bt };
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        VisualTypeConf tre = new VisualTypeConf();
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
        tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);

        return new VisualTypeConf[] { tabl, tre};
    }


    static final long serialVersionUID = -3735922270591531741L;
}
