/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package content.exercises;

import java.util.Random;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;

public class Hash_Linearprobe implements SimulationExerciseModel, ModelAnswerNames, ConfigureVisualType {

    // SEED??

    Table t;
    VirtualArray va;
    VirtualArray solution;
    static final int size = 19;
    static final int keySize = 17;
    static final int min = 100;
    static final int max = 999;
    static final int dublicates = 5;
    static final int moreDublicates = 3;
    private int table[] = new int[keySize];
    public static final boolean DEBUG = false;

    public Hash_Linearprobe() {
    }

    long seed = 1;

    /**
     * Get the value of seed.
     * @return value of seed.
     */
    public long getSeed() {
        return seed;
    }

    /**
     * Set the value of seed.
     * @param v  Value to assign to seed.
     */
    public void setSeed(long  v) {
        this.seed = v;
    }

    public FDT[] init() {
	// new random
        Random R = new Random(seed);
	// create random data

        for(int i = 0; i < keySize-dublicates; i++) {
            table[i] = Math.abs(R.nextInt()%(max-min))+min;
        }

        // Create dublicates
        for(int i = keySize-dublicates; i < keySize; i++) {
            
	    do {
		table[i] = table[(0+i+dublicates-keySize)%(dublicates-moreDublicates)];
		table[i] -= (R.nextInt(keySize))*size;
	    } while(table[i]<min || table[i] > max);
        }

        t = new Table(table);
        va = new VirtualArray();
        va.setObject(null, 0);
        va.setObject(null, size-1);
        return new FDT[] {t, va};
    }


    public FDT[] init(Object o) {
        return init();
    }

        public String[] getStructureNames() {
        return new String[] { "Stream of Keys", "Hash table" };
    }

    public String getDescription() {
        String s = "The hash function is h(k) = (k + 5) mod 19\n";
        return s;
    }

    public FDT[] solve() {
        solution = new VirtualArray();
        solution.setObject(null, 0);
        solution.setObject(null, size-1);
        Table tbl = (Table)getInitialStructures()[0];
        Animator a = Animator.getActiveAnimator();
        for(int i = 0; i < tbl.size(); i++) {
            a.startOperation();
	    probe(table[i]);
            a.endOperation();
        }
        return new FDT[] { solution };
    }



    public FDT[] makeModelAnswer() {
	return solve();
    }


    public FDT[] getInitialStructures() {
        VirtualArray tmp = new VirtualArray();
        tmp.setObject(null, 0);
        tmp.setObject(null, size-1);
        return new FDT[] { new Table(table), tmp };
    }

    public FDT[] getSimulatedStructures() {
        return new FDT[] { t, va };
    }

    private void probe(int key) {
        int n = 0;
	// hash key
        int hashValue = (key + 5) % size;
        if (DEBUG)
	    System.out.println(this+ "i: " + hashValue);
        Object o;
        for (int i = 0; i < size; i++) {
            if (DEBUG) System.out.println(key+":"+((hashValue+i)%size));
            o = solution.getObject((hashValue+i)%size);
            if(o == null) {
                solution.setObject(new Key(key), (hashValue+i)%size);
                return;
            }
            n++;
        }
    }


    public String[] getModelAnswerNames() {
        String[] s = { "Hash Table" };
        return s;
    }


    public FDT[] getAnswer() {
        return new FDT[] { va };
    }

    public SimulationExerciseModel getModelAnswer() {
        return this;
    }

    public VisualTypeConf[] conf() {
        VisualTypeConf tabl = new VisualTypeConf();
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
        tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        VisualTypeConf hashtabl = new VisualTypeConf();
        hashtabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
        hashtabl.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
        hashtabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
        hashtabl.enable("matrix.visual.VisualArrayComponent", VisualTypeConf.DROP_OPERATION);
        return new VisualTypeConf[] { tabl, hashtabl };
    }

    static final long serialVersionUID = -37657283333350211L;
}
