/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: Heap_Insert_Delete.java,v 1.20 2004/05/26 08:05:44 vkaravir Exp $*/

package content.exercises;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.util.RandomKey;

public class Heap_Insert_Delete extends AbstractSimulationExercise 
    implements SimulationExerciseModel, 
	       ModelAnswerNames, 
	       ConfigureVisualType, 
	       ButtonExercise {

    Table t;
    ExerHeap bt, bh;
    
    private String S = null;
    public static final boolean DEBUG = false;
    
    public Heap_Insert_Delete() {}


  public FDT[] init() {
    //t = new Table("KVEFGOQXNP");
    S = RandomKey.createNoDuplicateUppercaseRandomKey(new java.util.Random(seed),15);
    t = new Table(S);
    bt = new ExerHeap(15);
    FDT[] s = { t, bt };
    return s;
  }


  public FDT[] init(Object o) {
    return init();
  }


  public String[] getStructureNames() {
    String[] s = { "Stream of keys", "Heap" };
    return s;
  }

  public String getDescription() {
      return "";
  }

  public FDT[] solve() {
    bh = new ExerHeap(15);
    Animator a = Animator.getActiveAnimator();
    Table tbl = (Table)getInitialStructures()[0];
    for(int i = 0; i < tbl.size(); i++) {
      a.startOperation();
      bh.insert(tbl.getObject(i));
      a.endOperation();
    }
    for(int i = 0; i < 3; i++) {
      a.startOperation();
      bh.delete(null);
      a.endOperation();
    }
    FDT[] s = { bh };
    return s;
  }

  public FDT[] getInitialStructures() {
    return new FDT[] { new Table(S), new ExerHeap(15) };
  }

  public FDT[] getSimulatedStructures() {
      return new FDT[] { t, bt };
  }

  public String[] getModelAnswerNames() {
    String[] s = { "Binary Heap" };
    return s;
  }


  public FDT[] getAnswer() {
    return new FDT[] { bt };
  }

  public SimulationExerciseModel getModelAnswer() {
    return this;
  }

    public String[] buttonNames() {
        String[] s = {"Delete"};
        return s;
    }

    public String[] buttonCommands() {
        String[] s = {"reflectSelectedVisualType(reflectEDT(deleteRoot))"};
        return s;
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tabl = new VisualTypeConf();
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf tre = new VisualTypeConf();
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    tre.enable("matrix.visual.VisualKey", VisualTypeConf.POP_UP_MENU_OPERATION);
    tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.HIGHLIGHT_OPERATION);
    tre.enable("matrix.visual.VisualLayeredTreeComponent", VisualTypeConf.DROP_OPERATION);
    return new VisualTypeConf[] { tabl, tre };
  }

  static final long serialVersionUID = -7671756502477641250L;
}
