/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: InterpolationSearch.java,v 1.28 2004/05/26 08:05:44 vkaravir Exp $*/

package content.exercises;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.structures.FDT.*;
import matrix.structures.memory.VirtualArray;
import matrix.util.Note;

public class InterpolationSearch extends BinarySearch implements ModelAnswerNames,
                                                                 ConfigureVisualType {

  public static final boolean DEBUG = false;

  public InterpolationSearch() {
   //   charKeys = new char[(int)'Z' - (int)'A' + 1];
//      for(char i='A'; i<='Z'; i++) {
//        charKeys[i-'A'] = i;
//      }

  }

  public InterpolationSearch(int[] sortedKeys, int key) {
    super(sortedKeys,key);
  }


  public String[] getStructureNames() {
    String[] s = { "Table of Keys","List of keys visited in interpolation search"};
    return s;
  }

  public String getDescription() {
    return "Key to find: " +keyToSearch;
  }

  public String[] getModelAnswerNames() {
    String[] s = { "Table of keys" };
    return s;
  }


  protected int getMiddleOne(int left, int right) {

    if (left >= right) return left;
    int mid = left +  Math.abs( (keyToSearch - keyArray[left]) *
                                      (right - left) ) /
                           ( keyArray[right] - keyArray[left] );
    Note.out(this,"calc middle one=" + mid);
    return mid;
  }

  public FDT[] solve() {
    Note.out(this, "Solve");
    LinkedList modelList = (LinkedList)getInitialStructures()[1];
    LinkedList last = modelList;
    VirtualArray model = (VirtualArray)getInitialStructures()[0];
    Animator a = Animator.getActiveAnimator();
    int low = 0;
    int high = model.size() - 1;
    int mid = 0;
    int i = 0;
    while (keyArray[low] < keyToSearch && keyArray[high] >= keyToSearch) {
      a.startOperation();
      mid = getMiddleOne(low,high);
      if (DEBUG) {
        System.out.println("Iteration: "+i++);
        System.out.println("Low: "+low);
        System.out.println("High: "+high);
        System.out.println("Mid: "+mid);
        System.out.println("Key: "+keyArray[mid]);
      }
      if (model.getObject(mid) != null ) {
        last.setNext(last.getNewNode(model.getObject(mid)));
        last = last.getNext();
      }
      if( keyArray[ mid ] < keyToSearch)
        low = mid + 1;
      else if( keyArray[ mid ] > keyToSearch )
	high = mid - 1;
      else {
        a.endOperation();
	return new FDT[] { modelList.getNext() };  /* Found mid*/
      }
      a.endOperation();
    }
    /* not added to model answer
      //compare with the key at low
      a.startOperation();
      last.setNext(last.getNewNode(model.getObject(low)));
      last = last.getNext();
      a.endOperation();
    */

    Note.out(this, "Solved");
    return new FDT[] { modelList.getNext() };
  }

  public VisualTypeConf[] conf() {
    VisualTypeConf tabl = new VisualTypeConf();
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.HIGHLIGHT_OPERATION);
    tabl.enable("matrix.visual.VisualKey", VisualTypeConf.DRAG_OPERATION);
    VisualTypeConf lis = new VisualTypeConf();
    lis.enable("matrix.visual.VisualKey", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualListComponent", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.DROP_OPERATION);
    lis.enable("matrix.visual.VisualList", VisualTypeConf.HIGHLIGHT_OPERATION);
    return new VisualTypeConf[] { tabl, lis};
  }


  static final long serialVersionUID = 2428841166095544234L;
}
