/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Judgement.java,v 1.6 2004/05/25 13:27:59 kstalnac Exp $ */

package content.exercises;

import java.util.Vector;

/**
 * This class contains a criticism for an exercise, and is created
 * and returned by the judge() -method in Exercise interface.
 * @see Exercise
 */
public class Judgement {

  /** Points got from the exercise */
  private int[] points;
  private int maxPoints;
  private String[] names;
  private Vector notes = new Vector();
  private boolean complete = false;
  private int errorStateIndex = -1;
  
  /**
   * Constructs a new Judgement
   */
  public Judgement(int i) {
    points = new int[i];
    names = new String[i];
  }
  
  /**
   * Returns the points got from the exercise judged.
   * @return the points
   */
  public int[] getPoints() {
    return points;
  }
  
  /**
   * Sets the points got from the exercise judged.
   * @param i the points
   */
  public void setPoints(int i, int pos) {
    if(pos > -1 && pos < points.length)
      points[pos] = i;
  }
  
  public void setMax(int max) {
    maxPoints = max;
  }

  public int getMax() { return maxPoints; }

  public void setNames(String[] names) { this.names = names; }

  public String[] getNames() { return this.names; }
  
  public void setComplete() {
    complete = !complete;
  }
  
  public boolean isComplete() {
    return complete;
  }


  public void setErrorStateIndex(int value) {
    errorStateIndex = value;
  }


  public int getErrorStateIndex() {
    return errorStateIndex;
  }

  public boolean errorStateIndexSet() {
    return errorStateIndex >= 0;
  }

  
  public void addNote(Object note) {
    notes.addElement(note);
  } 
}








