/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*$Id: Jury.java,v 1.16 2004/05/26 08:05:44 vkaravir Exp $*/

package content.exercises;

import matrix.animation.Animator;
import matrix.structures.FDT.FDT;
import matrix.util.Note;

public class Jury {
  
    protected static Animator userAnimator = null;
    protected static Animator solverAnimator = null;
    protected static int userSolutionState;
    protected static FDT[] userSolution;
    protected static FDT[] modelSolution;
    
  //protected static final String END_OF_SOLUTION = "UserSolutionEnds";

  public static Judgement judge(SimulationExercise e, Animator a) {
    if(e instanceof SimulationExerciseSelfAssessment) {
	// Init
	String[] names = e.getStructureNames();
	int[] points = ((SimulationExerciseSelfAssessment)e).judge();
	// Create judgement with lenght of points-array
	if (names == null || points == null) return new Judgement(0);
	Judgement J = new Judgement(points.length);
	// Set names
	J.setNames(names);
	// Set points
	int gotPoints = 0;
	for (int i=0; i < points.length;i++) {
	    J.setPoints(points[i],i);
	    gotPoints+=points[i];
	}
	// Set max points
	int maxPoints = ((SimulationExerciseSelfAssessment)e).getMaxPoints();
	J.setMax(maxPoints);
	// Return the created judgement
	if (gotPoints == maxPoints)
	    J.setComplete();

	return J;

    } else if(e instanceof SimulationExerciseModel) {
      return judge(e, (SimulationExerciseModel)e, a);
    } else {
      Note.err(null, "Unable to solve exercise, no SimulationModelAnswer present");
      return new Judgement(0);
    }
  }

    public static Judgement judge(SimulationExercise e, 
				  SimulationExerciseModel mod, 
				  Animator a) {
      if(e != null)
	return judge(e.getAnswer(), a, mod);
      else return new Judgement(0);
    }

  /** 
   * For each FDT in the user solution of a <code>SimulationExercise</code> the number of 
   * states corresponding to correct states in the model answer is counted.
   * The number of correct states for each FDT is returned in the instance of <code>Judgemet</code>
   * returned by this method. The current state of the user animator a is the last
   * state judge will evaluate.
   *
   * @param user an array of the user's FDTs
   * @param mod the corresponding model answer for this exercise
   * @param a the animator of the user exercise
   */

  public static synchronized Judgement judge(FDT[] user, Animator a, SimulationExerciseModel mod) {
      // Begin kludge
      // ExerciseFrame in ui/prototype needs fixing
    if(mod instanceof SimulationExerciseSelfAssessment) {
	// Init
	String[] names = mod.getStructureNames();
	int[] points = ((SimulationExerciseSelfAssessment)mod).judge();
	// Create judgement with lenght of points-array
	if (names == null || points == null) return new Judgement(0);
	Judgement J = new Judgement(points.length);
	// Set names
	J.setNames(names);
	// Set points
	int gotPoints = 0;
	for (int i=0; i < points.length;i++) {
	    J.setPoints(points[i],i);
	    gotPoints+=points[i];
	}
	// Set max points
	int maxPoints = ((SimulationExerciseSelfAssessment)mod).getMaxPoints();
	J.setMax(maxPoints);
	// Return the created judgement
	if (gotPoints == maxPoints)
	    J.setComplete();
	return J;
    }

    //Note.out(null, "judge start");

	// sanity check
	if(mod == null || a == null || user == null) {
	  Note.err(null, "judge: null argument in judge, cannot judge exercise");
	  return new Judgement(0);
	}

	userAnimator = a; // userAnimator is a class variable
	solverAnimator = new Animator();
	int modelCount = 0; //number of correct states is modelSolution
	int userCount = 0;  //number of states in user's sulution
	int correctCount = 0; // number of correct states in user solution
	//int userState = 0; //offset from userAnimator. Null state is currents state

	Animator.setActiveAnimator(solverAnimator);
	modelSolution=mod.solve();


	userSolution = user; // userSolution is a class variable
	
	if (mod instanceof JudgeBlocks){
    
	    Judgement result = new Judgement(1);
 
	    if(modelSolution.length != userSolution.length) {
		Note.out(null, "userSolution and modelSolution have different number of structures, stopping judgement");
		return result;
	    }

	    //Note.out(null, "rewinding model solution");
	    modelCount = rewindAnimator(solverAnimator);
	    //Note.out(null, "rewinding user solution");
	    // GUI handles placing the beginning state of the user animator to the right place 
	    
	    // the states of the user animator are evaluated only until the current state.
	    // If there are any more states in the user animator, they are removed
	    
	    // Panu commented this out 2003-01-21
	    //Stack redoStack = userAnimator.disposeRedo(); 
	    
	    userCount = rewindAnimator(userAnimator);
	    Note.out(null, "starting judgement user: "+userCount+" model: "+modelCount);
	    int oldUserState = 0; //state of user solution in last iteration
	    
	    //    rewindAnimator(solverAnimator);
	    //rewindAnimator(userAnimator);
	    //userState = -userCount;
	    userSolutionState = 0;
	    correctCount = 0;
	    while(!(allEqual(userSolution,modelSolution)) && solverAnimator.hasNextOperation()) {
		solverAnimator.redo();
		modelCount--;
	    }
	    	    
	    if(!(allEqual(userSolution,modelSolution)) || modelCount == 0) {
		Note.out(null, "No state matching the starting state of the user animator found");
		result.setPoints(0, 0);
		
	    }


	    for (int i=0; i<modelCount; i++) {
		solverAnimator.redo();
		//Note.out(null, "Comparing states");
		oldUserState = userSolutionState;
		
		while(!(allEqual(userSolution,modelSolution)) && userAnimator.hasNextOperation() ) {
		    userSolutionState++;
		    userAnimator.redo();
		}
		if (allEqual(userSolution,modelSolution)){
		    //Note.out(null,"state "+i+" correct");
		    // Added by Harri 11-08-2003
		    // all states must in correct order
		    if (i==correctCount)
			correctCount++;
		}
		else {
		    //Note.out(null,"state " + i + " wrong");
		    if (!result.errorStateIndexSet()) {
			result.setErrorStateIndex(i);
		    }
		    
		    for(; userSolutionState > oldUserState; userSolutionState--)
			userAnimator.undo();
		}
		
	    }
	    while(userAnimator.hasNextOperation()) {
		userAnimator.redo();
	    }

	    result.setPoints(correctCount, 0);	    
	    
	    if(isFinished()) {
		result.setComplete();
		Note.out(null, "exercise is complete");
	    }

	    forwardAnimator(userAnimator);
	    result.setMax(modelCount);

	    String[] allNames = new String[]{"NEVER USED!"};//mod.getStructureNames();
	    result.setNames(allNames);
	    return result;
	}

  
	else{
	    
	    //NORMAL MODE (not JudgeBlocks)

	    // TODO how to custom Judgement for different exercises
	    Judgement result = new Judgement(modelSolution.length); 
	    if(modelSolution.length != userSolution.length) {
		Note.out(null, "userSolution != modelSolution, stopping judgement");
		return result;
	    }
	    //Note.out(null, "rewinding model solution");
	    modelCount = rewindAnimator(solverAnimator);
	    //Note.out(null, "rewinding user solution");
	    // GUI handles placing the beginning state of the user animator to the right place 
	    
	    // the states of the user animator are evaluated only until the current state.
	    // If there are any more states in the user animator, they are removed
	    
	    // Panu commented this out 2003-01-21
	    //Stack redoStack = userAnimator.disposeRedo(); 
	    
	    userCount = rewindAnimator(userAnimator);
	    Note.out(null, "starting judgement");
	    int oldUserState = 0; //state of user solution in last iteration
	    
	    // For-loop executed for each FDT in modelSolution
	    for(int j = 0; j < modelSolution.length; j++) {
		rewindAnimator(solverAnimator);
		rewindAnimator(userAnimator);
		//userState = -userCount;
		userSolutionState = 0;
		correctCount = 0;
		while(!(userSolution[j].equals(modelSolution[j])) && solverAnimator.hasNextOperation()) {
		    solverAnimator.redo();
		}
		if(!(userSolution[j].equals(modelSolution[j]))) {
		    Note.out(null, "No state matching the starting state of the user animator found");
		    result.setPoints(0, j);
		    continue;
		}
		for (int i=0; i<modelCount; i++) {
		    solverAnimator.redo();
		    //Note.out(null, "Comparing states");
		    oldUserState = userSolutionState;
		    if (find(j)) {
			//Note.out(null,"state "+i+" correct");
			// Added by Harri 11-08-2003
			// all states must in correct order
			if (i==correctCount)
			    correctCount++;
		    }
		    else {
			//Note.out(null,"state " + i + " wrong");
			if (!result.errorStateIndexSet()) {
			    result.setErrorStateIndex(i);
			}
			
			for(; userSolutionState > oldUserState; userSolutionState--)
			    userAnimator.undo();
		    }
		    
		}
		while(userAnimator.hasNextOperation()) {
		    userAnimator.redo();
		}
		// XXX TODO: what to do if there are states beyond the last correct state.
		/*
		  if(!userSolution[j].equals(modelSolution[j]))
		  if(correctCount == modelCount * modelSolution.length)
		  correctCount = correctCount / 2;
		*/
		result.setPoints(correctCount, j);
		
	    }
	    if(isFinished()) {
		result.setComplete();
		Note.out(null, "exercise is complete");
	    }
	    //Note.out(null, "judge end");
	    forwardAnimator(userAnimator);
	    
	    // Panu commented this out 2003-01-21
	    //userAnimator.setNewRedoStack(redoStack);
	    
	    result.setMax(modelCount);
	    String[] allNames = mod.getStructureNames();
	    /*
	      String[] names = new String[modelSolution.length];
	      System.arraycopy(allNames, allNames.length-modelSolution.length, names, 0, names.length);
	    */
	    result.setNames(allNames);
	    return result;
	}
    }

    protected static  boolean isFinished() {
	forwardAnimator(userAnimator);
	forwardAnimator(solverAnimator);
	for(int i = 0; i < userSolution.length; i++) {
	    if(!(userSolution[i].equals(modelSolution[i])))
		return false;
	}
	return true;
    }

    /** 
     * Rewinds user's solution to the beginning. Selected animator is not changed 
     * @return number of redo operations to restore animator's state
     */
    protected static int rewindAnimator(Animator a) {
	int count = 0;
	while(a.hasPreviousOperation()) { 
	    a.undo();
	    count++;
	}
	return count;
    }

    /** 
     * Go to the end of the given animator. Selected animator is not changed 
     * @return number of undo operations to restore animator's state
     */
    protected static int forwardAnimator(Animator a) {
	int count = 0;
	while(a.hasNextOperation()) {	    
	    a.redo();
	    count++;
	}
	return count;
    }
    /** 
   * Search model solution's current state from user's solution. Search starts
   * from position defined by Animator
   */
    protected static boolean find(int j) {
	while(!(userSolution[j].equals(modelSolution[j])) && userAnimator.hasNextOperation() ) {
	    userSolutionState++;
	    userAnimator.redo();
	}
	return userSolution[j].equals(modelSolution[j]);

    }

    private static boolean allEqual(FDT[] a, FDT[] b){
	for (int i=0; i<a.length; i++)
	    if (a[i].equals(b[i]) == false)
		return false;
	return true;
    }

}
