/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Assignment.java,v 1.13 2004/05/26 08:05:48 vkaravir Exp $ */

package matrix.animation;

import matrix.structures.memory.MemoryStructure;
import matrix.visual.VisualType;

/**
 * Animated assignment to be stored in Animator
 */
public class Assignment implements AnimatedOperation {
    private MemoryStructure mem;
    private Object newData, oldData;
    private int index;

    /** Assignment: memory[index] = data
     * Stores the old and new values of memory[index] for backward (undo)
     * and forward (redo) animations.
     */
    public Assignment(MemoryStructure mem, Object data, int index) {
        this.mem = mem;
        this.oldData = mem.getMem(index);
        this.newData = data;
        this.index = index;
    }

    /** Reset the previous data of this assignment. */
    public void undo() {
        //Note.out(this, "Undoing memory structure " + mem + " of class " + (mem == null ? null : mem.getClass().getName()));
        mem.setMem(index, oldData);
        //Note.out(this, mem + " M["+index+"]="+oldData);
        // Give a visualisation hint for visualiser
        VisualType.touch(oldData);
        //Note.out(this, "UNDONE");
    }

    /** ReAssign data. */
    public void redo() {
        mem.setMem(index, newData);
        //Note.out(this, array.memory + " M["+index+"]="+newData);
        // Give a visualisation hint for visualiser
        VisualType.touch(newData);
    }
    /*
    public String toString() {
        String s = "M[" + index + "] = " + newData + "(" + oldData + ")";
        if ((newData != null) && (!(newData instanceof Integer)))
            s += " of " + newData.getClass();
        return s;
    }
    */
    public String toString() {
        return "M[" + index + "] = " + newData;
    }

    static final long serialVersionUID = 2694150148166825722L;
}
