/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Picker.java,v 1.28 2004/05/26 08:05:49 vkaravir Exp $ */

package matrix.simulation;

import java.awt.*;

import matrix.structures.FDT.FDT;
import matrix.util.*;
import matrix.visual.*;

public class Picker extends Container {
    /** st is the FDT that corresponds to the representation to be moved. */
    FDT st;
    /** orig is the local component and the vt is the copy of it. */
    VisualType vt, orig;
    /** Point p is the position in which the pick occured. */
    Point p;
    /** Application is the container this picker is attached to. */
    Application f;
    /** (dx,dy) is the cursor position within the picked object in which
     ** the pick occured. */
    int dx, dy;
    /** local == true if we are moving some local component
     ** such as reference and not the copy of it. */
    boolean local;

    public Picker() {}

    public int getDX() {
	return dx;
    }

    public int getDY() {
	return dy;
    }

    public VisualType getPick() { return vt; }

    public VisualType getOrig() { return orig; }

    public Dimension getMinimumSize() {
        return getPreferredSize();
    }

    private Dimension mySize = new Dimension(0,0);

    public Dimension getPreferredSize() {
        return mySize;
    }

    Image offscreen;
    int ix,iy;


    public synchronized VisualType pick(VisualType t,
                                        Point p,
                                        int dx,
                                        int dy) {

        this.f = t.getApplication();
        this.p = p;
        this.dx = dx;
        this.dy = dy;
        this.orig = t;

        Dimension size = getPreferredSize();
        this.setSize(size);

        /* Get moving part: the picked object may determine which
           representation it would give to be the moving part of it. */
        vt = t.getMovingPart();


        if (vt == null) { // getMovingPart not determined
            st = t.getStructure(); // create a copy of the original structure
            vt = RepresentationFactory.getRepresentation(t);
            local = false;
        } else {
            local = true;
        }
        add(vt); // add the representation into this picker

        vt.validate();
        vt.setBounds(p.x, p.y, vt.width, vt.height);
        vt.doLayout();

        Dimension mySize = f.getSize();
        if (!local) setBounds(1,1,mySize.width,mySize.height);

        // reset offscreen
        ix = iy = 0;
        offscreen = null;

	disableMouseEvents(vt);

        return vt;
    }

    private static void disableMouseEvents(VisualType c) {
        c.setMask(AWTEvent.MOUSE_EVENT_MASK, false);
        c.setMask(AWTEvent.MOUSE_MOTION_EVENT_MASK, false);

	for(int i=c.getComponentCount()-1; i>=0; i--) {
	    Component ch=c.getComponent(i);
	    if (ch instanceof VisualType)
		disableMouseEvents((VisualType)ch);
	}
    }

    public synchronized void drag(int x, int y) {
        if (offscreen != null) {
            ix = x - dx; iy = y - dy;
        }
        if (local)
            Note.err(this, "drag(): Should not happend because "+
                     "local components should override the drag by themself");
        else if(vt != null) 
	    vt.setLocation(ix+p.x, iy+p.y);
    }

    public void drop() {
        removeAll();
        offscreen = null;
    }

    public synchronized void paint(Graphics g) {
        if (g instanceof PrintGraphics)
            return;

        if (offscreen == null) {
            offscreen = createImage(getSize().width, getSize().height);
            super.paint(offscreen.getGraphics());
            return;
        }
        //Note.out(this, "osh ="+offscreen.getHeight(null));
        //Note.out(this, "osw ="+offscreen.getWidth(null));
        g.drawImage(offscreen, ix, iy, null);

        //super.paint(g);
        //Rectangle r = getBounds();
        //g.drawRect(1,1,r.width-2,r.height-2);
    }

    static final long serialVersionUID = 7531996408147136964L;
}
