/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: CDT.java,v 1.2 2004/05/25 13:28:07 kstalnac Exp $ */

package matrix.structures.CDT;

/**
 * This is the fundamental interface for all abstract data types.
 * It defines the basic operations for such data types. In
 * Matrix an abstract data types include all data types for which
 * there are legal insert and delete operations. For example
 * dictionaries, priority queues, stacks, deques, etc. are
 * abstract data types.
 *
 * An CDT (Conceptual data type) is a wrapper class that moderates
 * the outside access to the underlying (FDT) data structure. (for example
 * the <code>BinSearchTree</code> -class is a wrapper that holds
 * a binary tree inside it. The <code>BinSearchTree</code>
 * moderates the outside access to the underlying binary tree
 * and handles insertion and deletion from the tree in a manner
 * that keeps the binary search tree property true. Thus, as long as
 * all data structure wrapped in the CDT is handled only through the
 * CDT interface, the CDT works properly.
 *
 * An CDT does not necesserily need to implement any visualisation
 * interfaces. Matrix can automatically find the proper visualisation
 * for the underlying (FDT) data structure.
 *
 * For an CDT, the getElement() -method (inherited from FDT interface)
 * should return the source of the underlying data structure. For
 * trees, this is the root of the tree and for graphs the source
 * vertex of traversing algorithms.
 *
 * Insert and delete should return the CDT itself, or the new CDT if
 * the data structure is encased in a new wrapper after some operation.
 *
 */
import matrix.structures.FDT.FDT;

public interface CDT extends FDT {

    /**
     * Inserts a new object to this CDT.
     * @param o the Object to be inserted
     * @return the CDT wrapper of the data structure after insertion
     */
    public FDT insert(Object o);

    /**
     * Deletes the the parameter object from this CDT.
     * @param o the Object to be deleted
     * @return the CDT wrapper of the data structure after deletion
     */
    public FDT  delete(Object o);


    /** obsolete method. */
    public Object search(Object o);

    static final long serialVersionUID = -6333960859382490934L;
}
