/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.CDT.probe;

import java.util.TreeSet;

import matrix.structures.FDT.probe.BinaryTree;
import matrix.structures.memory.Key;

/**
 * A Faulty Binary search tree that can be used for demonstration
 * purposes. Dublicates are inserted to left whereas in deletion
 * replacing key is taken from the right branch. Therefore the tree is
 * not working correctly.
 *
 * @author Petri Tenhunen
 */
public class FaultyBinSearchTree extends BinSearchTree {

    
    /** Constructs a new BinSearchTree */
    public FaultyBinSearchTree() {
        super(null);
    }
    
    /**
     * Constructs a new BinSearchTree
     * @param o the key to the root node
     */
    public FaultyBinSearchTree(Object o) {
	super(o);
    }

    /**
     * Examine if the tree is broken. Tree is broken if a key is
     * simultaneusly in the left and right subtrees of some node 
     */
    public boolean isBroken() {
	return isBroken((BinaryTree)getElement());
    }

    
    protected boolean nodeIsBroken(BinaryTree tree) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null)
	    return false;

	return nodeIsBrokenRightHelp(tree.getRight(), key) ||  nodeIsBrokenLeftHelp(tree.getLeft(), key);
    }
    
    protected boolean nodeIsBrokenRightHelp(BinaryTree tree, Key thisKey) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null)
	    return false;
	if (key.leq(thisKey))
	    return true;
	return (nodeIsBrokenRightHelp(tree.getLeft(), thisKey) || nodeIsBrokenRightHelp(tree.getRight(), thisKey));
    }

    protected boolean nodeIsBrokenLeftHelp(BinaryTree tree, Key thisKey) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null)
	    return false;
	if (key.gt(thisKey))
	    return true;
	return (nodeIsBrokenLeftHelp(tree.getLeft(), thisKey) || nodeIsBrokenLeftHelp(tree.getRight(), thisKey));
    } 
    
    protected boolean isBroken(BinaryTree tree) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null)
	    return false;
	
	if ( nodeIsBroken(tree) ) {
	    return true;
	} else
	    return (isBroken(tree.getLeft()) || isBroken(tree.getRight()));
    }
	
	


    protected void getAllKeys(BinaryTree tree, TreeSet result) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null) return;
	
	BinaryTree left = tree.getLeft();
	BinaryTree right = tree.getRight();
	result.add(key);
	if (left != null)
	    getAllKeys(left, result);
	if (right != null)
	    getAllKeys(right, result);
    }
	

    /**
     * Helper function for delete.  <p> This makes this Searchtree to
     * be faulty. Replacing key is taken from the right branch but the
     * <code>insertHelp</code> inserts duplicates to the left.
     *
     * @param k the key to be deleted
     * @param tree the subtree currently being examined
     */
    protected BinaryTree deleteHelp(Key k, BinaryTree tree) {
	Key key = tree != null ? (Key)tree.getElement() : null;
	if (key == null) return null;
	if(k.equals(key)) {  // found the node to be deleted
	    BinaryTree left = tree.getLeft();
	    BinaryTree right = tree.getRight();
	    
	    if(left == null)
		tree = tree.getRight();
	    else if(right == null)
		tree = tree.getLeft();
	    else {
		Key tmp = getSmallest(right);
		tree.setElement(tmp);
		tree.setRight(deleteSmallest(right));
	    }
	} else if(k.lt(key))
	    tree.setLeft(deleteHelp(k, tree.getLeft()));
	else
	    tree.setRight(deleteHelp(k, tree.getRight()));
	return tree;
    }

    /**
     * Returns the smallest key in a subtree
     * @param tree the subtree currently being examined
     * @return the smallest key
     */
    protected Key getSmallest(BinaryTree tree) {
	if(tree.getLeft() == null)
	    return (Key)tree.getElement();
	else
	    return getSmallest(tree.getLeft());
    }

    /**
     * Deletes the smallest item in a subtree.
     * @param tree the subtree currently being examined
     * @return the tree after deletion
     */
    protected BinaryTree deleteSmallest(BinaryTree tree) {
	if(tree.getLeft() == null) {
	    return tree.getRight();
	} else {
	    tree.setLeft(deleteSmallest(tree.getLeft()));
	    return tree;
	}
    }


    static final long serialVersionUID = 490931325492135329L;
}
