/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Stack.java,v 1.5 2004/05/26 08:05:51 vkaravir Exp $ */

package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;

public class Stack extends Table implements CDT {
    
    int index;
    VirtualArray virtualArray;

    /**
     * Constructor
     *
     */
    public Stack() {
	super("    ");
	index = 0;
	virtualArray = new VirtualArray();
	virtualArray.setObject(new Integer(index), 0); 
    }

    public Stack(int i) {
	super(" ");
	index = 0;
	virtualArray = new VirtualArray();
	virtualArray.setObject(new Integer(index), 0);
	if(i > 0) {
	    for(int j = 0; j < i; j++) {
		setObject(new Key(" "), j);
	    }
	}
    }

    public Object getElement() {	
	VirtualArray va = new VirtualArray();
	for(int i=0; i <= getLast(); i++) {
	    va.setObject(getObject(i), i);
	}
	return va;
    }
    

    /**
     * Inserts key to the top of the stack.
     *
     * @param o key to be inserted
     * @return stack after insertion
     */
    public FDT insert(Object o) {
	index = ((Integer)virtualArray.getObject(0)).intValue();
	if (index >= getLast()) {
	    if(o instanceof FDT)
		setObject(o, index);
	    else
		setObject(new Key(o), index);
	    setLast(getLast()*2 + 1);
	}
	else {
	    if(o instanceof FDT)
		setObject(o, index);
	    else 
		setObject(new Key(o), index);
	}
	index++;
	virtualArray.setObject(new Integer(index), 0);
	return getArray();
    }


    /**
     * Deletes the given key and all the keys before it from the stack.
     *
     * @param o key to be deleted
     * @param stack after the deletion
     */
    public FDT delete(Object o) {
	boolean isInStack = false;
	boolean hasOriginalInStack = false;
	for(int i = 0; i < ((Integer)virtualArray.getObject(0)).intValue(); i++) {
	    if(o instanceof Key){
			if(getObject(i) == ((Key)o).getOrig()) {
		    	isInStack = true;
		    	hasOriginalInStack = true;
	    	}
	    	else {
				if(getObject(i) == o) {
					isInStack = true;
				}
	    	}
		} else {
			if(getObject(i) == o)
				isInStack = true;
		}
	}
	if(isInStack) {
	    if((o instanceof Key) && hasOriginalInStack) {
			while(!(getObject(((Integer)virtualArray.getObject(0)).intValue() - 1) == ((Key)o).getOrig())){
		    	pop(o);
			}
			pop(o);
			return getArray();
	    }
	    else {
			while(!(getObject(((Integer)virtualArray.getObject(0)).intValue() - 1) == o)){
		    	pop(o);
			}
			pop(o);
			return getArray();
	    }
	}
	return getArray();
    }

    /**
     * Removes the top key from the stack.
     *
     * @param o can be anything
     * @return stack after deletion
     */
    public FDT pop(Object o) {
	index = ((Integer)virtualArray.getObject(0)).intValue();
	if (index > 0) {
	    index--;
	    virtualArray.setObject(new Integer(index), 0);
	    setObject(null, index);
	}
	else {
	    setObject(null, 0);
	}
	return getArray();
    }	
    
    public Object search(Object o) {
	return getArray();
    }
    
    /**
     * Returns the top key of the stack.
     *
     * @return key from the top of the stack
     */
    public Object getTopOfStack() {
	index = ((Integer)virtualArray.getObject(0)).intValue();
	if(index > 0)
	    return getObject(index - 1);
	else
	    return getObject(0);
    }

    public int getStackSize(){
	return index;
    }

    public boolean isEmpty(){
	return index == 0;
    }

    static final long serialVersionUID = 7919275024495283856L;
}
