/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: StackTopVisible.java,v 1.4 2004/05/26 08:05:52 vkaravir Exp $ */

package matrix.structures.CDT.probe;

import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.probe.Table;
import matrix.structures.memory.*;

public class StackTopVisible extends Table implements CDT {
    
    int index;
    VirtualArray next;
    VirtualArray stackElements;

    /**
     * Constructor
     *
     */
    public StackTopVisible() {
	super(" ");
	index = 0;
	next = new VirtualArray();
	next.setObject(new Integer(index), 0);
	stackElements = new VirtualArray();
	stackElements.setObject(getObject(index), index);
    }


    public Object getElement() {	
	VirtualArray va = new VirtualArray();
	for(int i=0; i <= getLast(); i++) {
	    va.setObject(getObject(i), i);
	}
	return va;
    }
    

    /**
     * Inserts key to the top of the stack.
     *
     * @param o key to be inserted
     * @return stack after insertion
     */
    public FDT insert(Object o) {
	index = ((Integer)next.getObject(0)).intValue();
	
	if(o instanceof FDT) {
	    setObject(o, 0);
	    stackElements.setObject(o, index);
	}
	else {
	    setObject(new Key(o), 0);
	    stackElements.setObject(new Key(o), index);
	}

	index++;
	next.setObject(new Integer(index), 0);
	return getArray();
    }


    /**
     * Deletes the given key from the stack.
     *
     * @param o key to be deleted
     * @param stack after the deletion
     */
    public FDT delete(Object o) {
	pop(o);
	return getArray();
    }

    /**
     * Removes the top key from the stack.
     *
     * @param o can be anything
     * @return stack after deletion
     */
    public FDT pop(Object o) {
	index = ((Integer)next.getObject(0)).intValue();
	if (index > 0) {
	    index--;
	    stackElements.setObject(null, index);
	    next.setObject(new Integer(index), 0);
	    if(index > 0)
		setObject(stackElements.getObject(index - 1), 0);
	    else
		setObject(null, 0);
	}
	else {	    
	    setObject(null, 0);
	    stackElements.setObject(null, 0);
	}
	return getArray();
    }	
    
    public Object search(Object o) {
	return getArray();
    }
    
    /**
     * Returns the top key of the stack.
     *
     * @return key from the top of the stack
     */
    public Object getTopOfStack() {
	return getObject(0);
    }

    public int getStackSize(){
	return index;
    }

    public boolean isEmpty(){
	return index == 0;
    }

    static final long serialVersionUID = 789450193337119783L;
}
