/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.EDT;
import java.awt.Color;

import matrix.decoration.*;
import matrix.structures.FDT.FDT;
import matrix.structures.memory.*;
import matrix.util.Note;


/**
 * A key that can be colored using color codes in java.awt.Color. 
 * Color changes of default background color are animated.
 *
 * @author Panu Silvasti
 */

public class ColorableKey extends Key implements Styled, StyleSheet {

    private StyleSheet ss =  new StyleSheetAdapter() {
	    public Color getVisitedPenColor() { return Color.red; }
	    public Color getVisitedBorderColor() {return ColorableKey.this.getDefaultBorderColor();}
	    
	};
    
    public ColorableKey(String s) {
	super(s);
    }

    public ColorableKey(Object o) {
	super(o);
    }



    public boolean equals(Key k) {
	if (k instanceof ColorableKey) {
	    if (getDefaultBackgroundColor().equals(((ColorableKey)k).getDefaultBackgroundColor()))
		return super.equals(k);
	    else
		return false;	    
	} else {
	    return false;
	}
    }


    
    public boolean equals(Object k) {
	if (k instanceof ColorableKey) {
	    if (getDefaultBackgroundColor().equals(((ColorableKey)k).getDefaultBackgroundColor()))
		return super.equals(k);
	    else
		return false;	    
	} else {
	    return false;
	}
    }
    


    public StyleSheet getStyleSheet() {
	return this;
    }



    // a helper method for querying the attribute color
    private Color getColor(String s) {
	int val = 0;
	try {
	    val = Integer.valueOf(s).intValue();
	} catch (NumberFormatException e) {
	    Note.err(this, e.getMessage());
	    return null;
	}
	return new Color(val);
    }
    


    private VirtualArray defaultBackgroundColor = new VirtualArray(1);

    public void setDefaultBackgroundColor(Color c) {
	defaultBackgroundColor.setObject(c.getRGB()+"", 0);
    }

    

    public Color getDefaultBackgroundColor() {
	Object colorStr = defaultBackgroundColor.getObject(0);
	if (colorStr == null || colorStr.toString().trim().equals(""))
	    return ss.getDefaultBackgroundColor();
	else {
	    return getColor(colorStr.toString());
	}
    }



    public void setFocusedBackgroundColor(Color c) {
	ss.setFocusedBackgroundColor(c);
    }


    public void setVisitedBackgroundColor(Color c) {
	ss. setVisitedBackgroundColor(c);
    }


    public void setVisitedFocusedBackgroundColor(Color c) {
	ss.setVisitedFocusedBackgroundColor(c);
    }


    public Color getFocusedBackgroundColor() {
	return ss.getFocusedBackgroundColor();
    }


    public Color getVisitedBackgroundColor() {
	return ss.getVisitedBackgroundColor();
    }

    public Color getVisitedFocusedBackgroundColor() {
	return ss.getVisitedFocusedBackgroundColor();
    }


    public void setDefaultBorderColor(Color c) {
	ss. setDefaultBorderColor(c);
    }


    public void setFocusedBorderColor(Color c) {
	ss.setFocusedBorderColor(c);
    }


    public void setVisitedBorderColor(Color c) {
	ss.setVisitedBorderColor(c);
    }


    public void setVisitedFocusedBorderColor(Color c) {
	ss.setVisitedFocusedBorderColor(c);
    }
    

    public Color getDefaultBorderColor() {
	return ss. getDefaultBorderColor();
    }


    public Color getFocusedBorderColor() {
	return ss.getFocusedBorderColor();
    }


    public Color getVisitedBorderColor() {
	return ss.getVisitedBorderColor();
    }


    public Color getVisitedFocusedBorderColor() {
	return ss.getVisitedFocusedBorderColor();
    }
    

    public void setDefaultPenColor(Color c) {
	ss.setDefaultPenColor(c);
    }


    public void setFocusedPenColor(Color c) {
	ss.setFocusedPenColor(c);
    }


    public void setVisitedPenColor(Color c) {
	ss.setVisitedPenColor(c);
    }


    public void setVisitedFocusedPenColor(Color c) {
	ss.setVisitedFocusedPenColor(c);
    }
    

    public Color getDefaultPenColor() {
	return ss.getDefaultPenColor();
    }


    public Color getFocusedPenColor() {
	return ss.getFocusedPenColor();
    }


    public Color getVisitedPenColor() {
	return ss.getVisitedPenColor();
    }


    public Color getVisitedFocusedPenColor() {
	return ss.getVisitedFocusedPenColor();
    }


    public void setReferenceColor(FDT f, Color c) {
	ss.setReferenceColor(f, c);
    }


    public void setFocusedReferenceColor(FDT f, Color c) {
	ss.setFocusedReferenceColor(f, c);
    }


    public void setModifiedReferenceColor(FDT f, Color c) {
	ss.setModifiedReferenceColor(f, c);
    }


    public Color getReferenceColor(FDT f) {
	return ss.getReferenceColor(f);
    }


    public Color getFocusedReferenceColor(FDT f) {
	return ss.getFocusedReferenceColor(f);
    }


    public Color getModifiedReferenceColor(FDT f) {
	return ss.getModifiedReferenceColor(f);
    }


    //  --- implements InformationDecorator ---

    public boolean isInfoEnabled() { 
	return ss.isInfoEnabled();
    }


    public void setInfo(String info) { 
	ss.setInfo(info); 
    }


    public String getInfo() { 
	return ss.getInfo(); 
    }


    public int getWaitTime() { 
	return ss.getWaitTime();
    }


    public void setWaitTime(int t) { 
	ss.setWaitTime(t);
    }


    public boolean isLabelEnabled() { 
	return ss.isLabelEnabled(); 
    }


    public void setLabel(String s) {
	ss.setLabel(s);
    }


    public String getLabel() { 
	return ss.getLabel(); 
    }


    public boolean isReferenceLabelEnabled() { 
	return ss.isReferenceLabelEnabled();
    }


    public void setReferenceLabel(int i, String s) {
	ss.setReferenceLabel(i,s);
    }


    public String getReferenceLabel(int i) { 
	return ss.getReferenceLabel(i); 
    }
        
}
