/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.EDT;

import matrix.structures.CDT.probe.*;
import matrix.structures.FDT.*;
import matrix.structures.memory.Element;
import matrix.structures.other.Rotations;
import matrix.util.Note;

/**
 * @see RBTree
 *
 * @author Petri Tenhunen
 */

public class ExerRBTree extends RBTree {

  public class ExerRBNode extends RedBlackNode implements EDT {

    private int i = 0;

    /** default constructor for empty tree */
    public ExerRBNode() {
      super();
    }

    public Object getElement() {
        //System.out.println("Refresh:"+i++);
        //toggleColor();
        return super.getElement();
    }

      public void setElement(Object o) {
          //System.out.println("Set:"+i++);
          //toggleColor();
          super.setElement(o);
      }

    /** Constructor for tree with one element */
    public ExerRBNode(Element e) {
      super(e);
    }

    public String[] getAllowedMethods() {
      return new String[] {"toggleColor","rotateLeft","rotateRight"};
    }

    /** If the color of the node is red - set it black and vice versa */
    public void toggleColor() {
      setRed(!isRed());
    }

    public void rotateRight() {
      Rotations.rotateWithLeft_(this);
    }

    public void rotateLeft() {
      Rotations.rotateWithRight_(this);
    }

    public void doubleRotateLeft() {
      Rotations.rotateDoubleWithLeft_(this);
    }

    public void doubleRotateRight() {
      Rotations.rotateDoubleWithRight_(this);
    }


    public Tree getNewNode(Object key) {
      ExerRBNode tmp =  new ExerRBNode((Element)key);
      tmp.setRed(true);
      return tmp;
    }

  }

  /** Overriden insertion does nothing */
  public FDT insert(Object o) {
    Note.show(this,"Insert nodes by dragging them on the leaf.");
    return this;
  }

  public Tree getNewNode(Object key) {
    return new ExerRBNode((Element)key);
  }


  static final long serialVersionUID = -8279204555769956022L;
}
