/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Graph.java,v 1.11 2004/05/26 08:05:53 vkaravir Exp $ */

package matrix.structures.FDT;


/** Common interface for all graphs.
 * Let V (vertices) be a finite set, and
 * let E (edges) be the subset of V x V. Then
 * G = (V,E) is called a (directed) graph.
 * In addition, G = (V,E,W) is a weighted graph in which
 * each edge e in E has an integer valued weight.
 * An ordinary graph has weights w in {0,1}.
 * @author Ari Korhonen, Riku Saikkonen, Jussi Nikander
 * @version $Revision: 1.11 $
 */
public interface Graph extends FDT {

    /**
     * Returns the nodes in the graph.
     * Must return at least the nodes from which all of the graph can
     * be reached; can return all nodes.
     * @return a Vertex array containg the nodes of the graph
     */
    public Vertex[] getVertices();

    static final long serialVersionUID = 3068127734456542043L;
}
