/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: SimulationGraph.java,v 1.4 2004/05/25 13:28:12 kstalnac Exp $ */

package matrix.structures.FDT;

//import matrix.structures.memory.*;
//import matrix.structures.FDT.*;

/** Interface for dynamic graphs (graphs where the number of vertices may be changed).
 * The getElement() -method of a Dynamic graph should return the 
 * Vertex of the graph that is considered the root or source of the graph.
 * (ie. the Vertex where all graph traversing algorithms start their operation)
 * setElement is used to change this source Vertex.
 *
 * @author Riku Saikkonen, Ari Korhonen, Jussi Nikander
 * @version $Revision: 1.4 $
 */
public interface SimulationGraph extends Graph {

    /**
     * Sets the vertices of this graph.
     * @param v the new set of vertices
     */
    public void setVertices(Vertex[] v);

    /**
     * Inserts a new Vertex into the graph.
     * @param o the key of the new Vertex to be inserted
     */
    public void addVertex(Object o);


    /**
     * deletes a Vertex from the graph
     * @param v the vertex to be deleted
     */
    public void removeVertex(Vertex v);

}












