/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: StaticLinkedList.java,v 1.5 2004/05/25 13:28:13 kstalnac Exp $ */

package matrix.structures.FDT;

/**
 * Interface for iterator-type lists (a single, mutable object
 * containing a pointer to the current location) in Matrix.
 */
public interface StaticLinkedList extends FDT {
    /**
     * Inserts the param Object at the current position in the list.
     * @param item the Object to be inserted
     */
    public void insert(Object item);

    /**
     * Appends the param Object to the tail of the list.
     * @param item the Object to be inserted
     */
    public void append(Object item);

    /** Sets the current position to the first position of the list. */
    public void setFirst();

    /** Sets the current position to the next element of the list. */
    public void next();
    /**
     * Sets the current position to the param value.
     * @param pos the new current position.
     */
    public void setPos(int pos);

    /**
     * Returns the Element at the current position.
     * @return the Object at the current position
     */
    public Object currValue();

    /**
     * Tells if a list is empty or not.
     * @return true if the list is empty
     */
    public boolean isEmpty();

    /**
     * Tells if current position is in the list (usually called after
     * a next() to see if there are more elements).
     * @return true if the current position is in the list
     */
    public boolean isInList();

    static final long serialVersionUID = 9222917958363582017L;
}
