/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Tree.java,v 1.12 2004/05/25 13:28:14 kstalnac Exp $ */

package matrix.structures.FDT;

/**
 * Tree is the common interface for all FDTs that need a tree visualisation.
 * Matrix can visualise all FDTs that implement Tree as trees.
 */

public interface Tree extends FDT /* Graph */ {

    /**
     * Returns the number of child nodes this tree node has.
     * If leaf node has empty subtrees this method should
     * return the number of those subtrees
     * even though getSubTrees() returns empty table.
     * @return the number of subtrees
     */
    public int getSubTreeCount();

    /**
     * Return a Tree table containing the subtrees of this
     * tree node. Empty subtrees should return null.
     * @return a table of Trees
     */
    public Tree[] getSubTrees();

    static final long serialVersionUID = 6520428734018443270L;
}
