/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: BinTree.java,v 1.15 2004/05/26 08:05:53 vkaravir Exp $ */

package matrix.structures.FDT.probe;

// Binary Tree implementation

//import java.awt.Container;
import java.util.Vector;

import matrix.structures.FDT.*;
import matrix.structures.memory.*;
import matrix.structures.other.Comparer;
import matrix.util.Note;

/**
 * This is an implementation of a binary tree. A binary tree is a
 * tree in which each node has two children (either of which can be null).
 */
public class BinTree implements BinaryTree, SimulationTree, Array, Vertex, Iterator {
    /** bt[0] is the key
        bt[1] is the left child
        bt[2] is the right child */
    private VirtualArray bt;

    public BinTree getParent(Object o) {
        return null;
    }

    /** construct a new empty <code>BinaryTree</code> */
    public BinTree() {
        bt = new VirtualArray();
    }

    /**
     * Constructs a new <code>BinaryTree</code>.
     * @param obj the contents of the root node
     */
    public BinTree(Object obj) {
        bt = new VirtualArray();
        bt.setObject(obj, 0);
    }

    public boolean equals(Object o) {
        if(o instanceof Tree)
            return Comparer.compareIdentical(this, (Tree)o);
        return false;
    }

    // FDT interface Methods
    public FDT getNewStructure() { return new BinTree(); }

    // CDT (not actually defined as an interface)

    // Return and set the element value
    public Object getElement() {
        return bt.getObject(0);
    }

    public void setElement(Object v) {
        //Note.out(this, "setElement " + v);
        bt.setObject(v, 0);
    }

    // Return and set the left child
    public BinaryTree getLeft() {
        return (BinaryTree)bt.getObject(1);
    }

    public void setLeft(BinaryTree p) {
        bt.setObject(p, 1);
    }

    // Return and set the right child
    public BinaryTree getRight() {
        return (BinaryTree)bt.getObject(2);
    }

    public void setRight(BinaryTree p) {
        bt.setObject(p, 2);
    }

    // Return true if this is a leaf node
    public boolean isLeaf() {
        return ((bt.getObject(1) == null) && (bt.getObject(2) == null));
    }

    // Array

    public Array getArray() { return bt; }

    public Object getObject(int index) {
        return bt.getObject(index);

    }
    public void setObject(Object o, int index) {
        if (index == 0)
            setElement(o);
        else
            Note.show(this, "Illegal operation");
    }

    public int getFirst() { return 0; }

    public int getLast() { return 2; } //bt.size()-1; }

    public void setFirst(int i) { bt.setFirst(i); }

    public void setLast(int i) { bt.setLast(i); }

    // Methods for interface Tree
    public int getSubTreeCount() {
        if (getLeft() != null || getRight() != null || getElement() != null)
          return 2;
        else
            return 0;
        //if ((getLeft() == null) && (getRight() == null)) return 0;
        //if ((getLeft() != null) && (getRight() != null)) return 2;
        //return 1;
    }

    public Tree[] getSubTrees() {
        Tree[] t = new Tree[2];
        t[0] = getLeft();
        t[1] = getRight();
        return t;
    }

    // Methods for interface Dynamic Tree

    public void setSubTree(Tree t, int child) {
        if ((t == null) || (t instanceof BinaryTree)) {
            BinaryTree bt = (BinaryTree)t;
            if (child == 0) setLeft(bt);
            else setRight(bt);
        } else Note.warning(this, "Parameter " + t + " not a binary tree");
    }

    public Tree getNewNode(Object key) {
        return new BinTree(key);
    }

  //public void insert(FDT s) { setElement(s); }

    public String toString() {
        if (getElement() instanceof Key)
            return getElement().toString();
        else {
            String s = "null";
            if (getElement() != null) s = getElement().getClass().toString();
            return super.toString() + "(" + s + ")";
        }
    }

    // Method(s) for interface Vertex
    public Vertex[] getSuccessors() {
        Tree[] t = getSubTrees();
        return new Vertex[] { (Vertex)t[0],(Vertex)t[1] };
    }

    // Method(s) for interface Graph
    /*
    public Vertex[] getVertices() {
        Vertex[] vLeft = null, vRight = null;
        if (getLeft() instanceof Graph)
            vLeft = ((Graph)getLeft()).getVertices();
        if (getRight() instanceof Graph)
            vRight = ((Graph)getRight()).getVertices();
        int size = 1;
        if (vLeft != null) size += vLeft.length;
        if (vRight != null) size += vRight.length;
        Vertex[] v = new Vertex[size];
        int cnt = 0;
        v[cnt++] = this;
        if (vLeft != null)
            for (int i=0; i<vLeft.length; i++)
                v[cnt++] = vLeft[i];
        if (vRight != null)
            for (int i=0; i<vRight.length; i++)
                v[cnt++] = vRight[i];
        return v;
    }
    */
    public void addSuccessor(Vertex v) {}

    public void removeSuccessor(Vertex v) {}

    // --- methods for interface iterator ---

    private Vector queue = null;

    public void reset() {
        queue = new Vector(10, 1);
        queue.addElement(this);
    }

    public boolean hasNext() {
        return (queue == null || queue.size() != 0);
    }

    public Object next() {
        if(queue == null) {
            queue = new Vector(10, 1);
            queue.addElement(this);
        }
        Tree tmp = (Tree)queue.elementAt(0);
        queue.removeElementAt(0);
        Tree[] c = tmp.getSubTrees();
        for(int i=0; i < c.length;i++)
            if(c[i] != null)
                queue.addElement(c[i]);
        return tmp.getElement();
    }


    static final long serialVersionUID = 449457555191564025L;
}
