/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Table.java,v 1.11 2004/05/26 08:05:55 vkaravir Exp $ */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.Iterator;
import matrix.structures.memory.*;

/**
 * A table of Objects that can be visualised as an Array in Matrix. It is possible
 * to create a "random table", which will randomize its keys each time it is
 * refreshed from the GUI. The random table uses three letter keys.
 */
public class Table extends VanillaTable implements Iterator {

    /* low and high indicate selected range of the table */
    private VirtualInteger low = new VirtualInteger(-1);
    private VirtualInteger high = new VirtualInteger(-1);
    /* boolean indicating if this is the first or second click 
       in */
    private boolean first = false;


    /** Constructs a new Table from the given <code>String</code>.
     * Each char in the string is stored in separate index of the
     * <code>Table</code>.
     * @param s the contens of the Table. */
    public Table(String s, int zero) {
        super(s, zero);
    }

    public Table(String s) {
        super(s, 0);
    }

    /** Constructs a new <code>Table</code> from a given <code>String</code> array.
     * Each string in the array is stored in a separate index of the <code>Table</code>.
     * @param s the contents of the Table.
     */
    public Table(String[] s, int zero) {
        super(s, zero);
    }

    public Table(String[] s) {
        super(s, 0);
    }

    public Table(Element[] e, int zero) {
        super(e, zero);
    }

    public Table(Element[] e) {
        super(e, 0);
    }

    public Table(int[] elements) {
        super(elements);
    }
    /**
     * Constructs a new <code>Table</code> with random three-letter keys.
     * @param size the size of the table
     */
    public Table(int size) {
        super(size);
    }

    //Iterator interface methods

    int index = 0;

    public void reset() {
        index = getFirst();
    }

    public boolean hasNext() {
        return index <= getLast();
    }

    public Object next() {
        return getObject(index++);
    }


    public void selectRange(int start, int end){
	low.assign(start);
	high.assign(end);
	first = false;
    }

    public void selectIndex(int index) {
	if (!first) {
	    low.assign(index);
	    high.assign(index);
	    //System.out.println("Selected range: low="+low.eval() + " and high="+high.eval());
	    first = true;	
	}
	else if (index < low.eval()) {
	    low.assign(index);
	    //System.out.println("Selected range: low="+low.eval() + " and high="+high.eval());
	    first = false;
	} 
	else {
	    high.assign(index);
	    //System.out.println("Selected range: low="+low.eval() + " and high="+high.eval());
	    first = false;
	}

    }


    public boolean isSelection() {
	return low.eval() != -1 && high.eval() != -1
	    && low.eval() <= high.eval();
    }
    

    public int selectionStart() {
	return low.eval();
    }


    public int selectionEnd() {
	return high.eval();
    }
    

    static final long serialVersionUID = -3417472670698508108L;
}
