/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VanillaTable.java,v 1.11 2004/05/26 08:05:56 vkaravir Exp $ */

package matrix.structures.FDT.probe;

import matrix.structures.FDT.FDT;
import matrix.structures.memory.*;
import matrix.util.RandomKey;

/**
 * A table of Objects that can be visualised as an Array in Matrix. It is possible
 * to create a "random table", which will randomize its keys each time it is
 * refreshed from the GUI. The random table uses three letter keys.
 */
public class VanillaTable extends VirtualArray implements FDT {

    //    public static final int random = 0;
    protected boolean rnd = false;

    /**
     * Constructs a new Table from the given <code>String</code>. Each char in
     * the string is stored in separate index of the <code>Table</code>.
     * @param s the contens of the Table.
     */
    public VanillaTable(String s, int zero) {
        super(zero);
        for (int i=0; i<s.length(); i++)
            setObject(new Key(s.substring(i,i+1)), i+zero);
    }

    public VanillaTable(String s) { this(s, 0); }

    /**
     * Constructs a new <code>Table</code> from a given <code>String</code> array.
     * Each string in the array is stored in a separate index of the <code>Table</code>.
     * @param s the contents of the Table.
     */
    public VanillaTable(String[] s, int zero) {
        super(zero);
        for(int i=0; i < s.length; i++)
            setObject(new Key(s[i]), i+zero);
    }

    public VanillaTable(String[] s) {
        this(s, 0);
    }

    public VanillaTable(Element[] e, int zero) {
        super(zero);
        for(int i=0; i < e.length; i++)
            setObject(e[i], i+zero);
    }

    public VanillaTable(Element[] e) {
        this(e, 0);
    }

    public VanillaTable(int[] elements) {
       rnd = true;
       for (int i = 0; i < elements.length; i++) {
        setObject(new Key(elements[i]+""), i);
       }
    }


    /**
     * Constructs a new <code>Table</code> with random three-letter keys.
     * @param size the size of the table
     */
    public VanillaTable(int size) {
        super();
        rnd = true;
	java.util.Random r = new java.util.Random(System.currentTimeMillis());
        if(size < 1) size = 1;
        for(int i = 0; i < size; i++)
	    setObject(new Key(RandomKey.createRandomKey(r)), i);
    }

    /**
     * Tells if this table random table or not.
     * @return true if table is random.
     */
    public boolean isRandomTable() {
        return rnd;
    }

    /**
     * Randomizes the contents of this table.
     */
    public void randomize() {
	matrix.util.Note.out(this, "RANDOMIZE");
	java.util.Random r = new java.util.Random(System.currentTimeMillis());
        for(int i = 0; i < this.size(); i++)
	    this.setObject(new Key(RandomKey.createRandomKey(r)), i);
    }

    // FDT interface Methods ????
    /*
    public FDT getNewStructure() {
        return new Table("");
    }
    */

    static final long serialVersionUID = -6755658672874936167L;
}
