/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.structures.adaptation;

import matrix.structures.FDT.LinkedList;

public class ListNodeAdapter extends Adapter implements LinkedList {
    private LinkedList next = null;
    private Object key;

    public ListNodeAdapter(Object key) {
        setElement(key);
    }

    public String getName() {
        return getElement().toString();
    }

    /**
     * @return the key of this Node.
     */
    public Object getElement() {
        return key;
    }

    /**
     * Sets the key of this Node.
     * @param o the new value of this Element
     */
    public void setElement(Object o) {
        key = o;
    }

    /**
     * Returns the next node in the list.
     * @return the successor of this list node
     */
    public LinkedList getNext() {
        return next;
    }

    /**
     * Sets the next node in the list.
     * @param l the new successor node
     */
    public void setNext(LinkedList l) {
        next = l;
    }

    /**
     * Tells if this node has a successor node.
     * @return true if there is a successor node, false otherwise
     */
    public boolean hasNext() {
        return (next != null);
    }

    /**
     * Creates a new list node.
     * @return the new node with specified key
     */
    public LinkedList getNewNode(Object key) {
        return new ListNodeAdapter(key);
    }

    static final long serialVersionUID = -6128757908972166808L;
}
