/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: Element.java,v 1.12 2004/05/25 13:28:19 kstalnac Exp $ */

package matrix.structures.memory;

import matrix.structures.FDT.FDT;
import matrix.util.Note;

public abstract class Element implements FDT {

    public void setElement(Object o) {
        Note.err(this,
                 "Element: Cannot perform setElement("+o+") for primitive");
    }

    public Object getElement() {
        return this;
    }

    public boolean eq(Element k) {
        if (k == null)
            return (this.cmpString() == "");
        if(k instanceof LexInf)
            return false;
        return (this.cmpString().equals(k.cmpString()));
    }

    public boolean lt(Element k) {
        if (k == null)
            return false;
        if(k instanceof LexInf)
            return true;
        return (this.toString().compareTo(k.cmpString()) < 0);
    }

    public boolean gt(Element k) {
        if (k == null)
            return (this.cmpString() != "");
        if(k instanceof LexInf)
            return false;
        return (this.toString().compareTo(k.cmpString()) > 0);
    }

    public boolean leq(Element k) {
        if(k instanceof LexInf)
            return true;
        return (this.eq(k) || this.lt(k));
    }

    public boolean geq(Element k) {
        if(k instanceof LexInf)
            return false;
        return (this.eq(k) || this.gt(k));
    }

    public abstract String cmpString();

    static final long serialVersionUID = 8312288442972609068L;
}
