/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.structures.memory;

/**
 * This is a singleton <code>Class</code> representing the
 * lexicographically infinity. It is greater than any other
 * Element.
 *
 * @author Jussi Nikander
 */
public class LexInf extends Element {

    private static Element inf = null;

    private LexInf() {
    }

    public static Element getInstance() {
        if(inf == null)
            inf = new LexInf();
        return inf;
    }

    public boolean eq(Element k) {
        return false;
    }

    public boolean lt(Element k) {
        return false;
    }

    public boolean gt(Element k) {
        return true;
    }

    public boolean geq(Element k) {
        return true;
    }

    public boolean leq(Element k) {
        return false;
    }

    public boolean equals(Object k) {
        if(k != this)
            return false;
        return true;
    }

    public String cmpString() {
        return "\uFFFF\uFFFF\uFFFF ...";
    }

    static final long serialVersionUID = 7773956070359944730L;
}
