/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.uitools;

import java.awt.Panel;
import java.awt.print.*;

import matrix.animation.Animator;

public class MatrixPageable implements Pageable {

    private int numOfPages = -1;
    private int processedPage = 0;
    private Animator animator;
    private MatrixFrame frame;
    private Panel sPanel;
    private PageFormat pf;

    public MatrixPageable(MatrixFrame f, PageFormat pf, boolean animSeq) {
	if (animSeq) { 
	    this.pf = pf;
	    frame = f;
	    animator = frame.getAnimator();
	    int steps = 1;
	    animator.rewind();
	    animator.end();
	    animator.rewind();
	    while(animator.hasNextOperation()) {
		animator.redo();
		steps++;
	    }
	    animator.rewind();
	    sPanel = frame.getStructurePanel();
	    numOfPages = steps;
	}
	else {
	    frame = f;
	    sPanel = frame.getStructurePanel();
	    this.pf = pf;
	    numOfPages = 1;
	}
    }

    public int getNumberOfPages() {
	return numOfPages;
    }

    public PageFormat getPageFormat(int pageIndex) throws IndexOutOfBoundsException {
	if (pageIndex > numOfPages || pageIndex < 0)
	    throw new IndexOutOfBoundsException();
	return pf;
    }

    public Printable getPrintable(int pageIndex) throws IndexOutOfBoundsException {
	if (pageIndex > numOfPages || pageIndex < 0)
	    throw new IndexOutOfBoundsException();
	if (numOfPages > 1) {
	    if (pageIndex > processedPage + 1 && animator.hasNextOperation()) {
		animator.redo();
		processedPage++;
	    } else {
		animator.rewind();
		int step = 0;
		while (step != pageIndex && animator.hasNextOperation()) {
		    animator.redo();
		    step++;
		}
		processedPage = pageIndex;
	    }
	    sPanel.validate();
	    frame.validate();
	}
	return new MatrixPrintable(sPanel, pageIndex);
    }
}
