/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package matrix.uitools;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.zip.GZIPOutputStream;

import matrix.animation.*;
import matrix.structures.FDT.FDT;
import matrix.util.*;
import matrix.util.export.*;
import matrix.util.io.*;

/**
 * MenuFrame is a subclass of MatrixFrame that contains all functionality required
 * by the "default" Menus included in the prototype application. All menu-related
 * stuff is here and not in MatrixFrame for two reasons. <p>
 *
 * <ol>
 * <li> If someone wishes to create an application where there are no menus, it
 * is possible to do so (and you do not need to use a class that contains
 * all the menu functionality</li>
 * <li> It is easier to maintain menu functionality when it has been separated
 * from the other functionality.</li>
 * </ol>
 */

public class MenuFrame extends MatrixFrame {

    public static final int DEFAULT_WIDTH = 10000;  
    public static final int DEFAULT_HEIGHT = 10000;

    private Menu fontMenu;

    // quick-hack version of history frame
    private boolean state = false;

    public MenuFrame() {
	super();
    }

    public MenuFrame(Animator a) {
	super(a);
    }

    public MenuFrame(StructurePanel p) {
	super(p);
    }

    public MenuFrame(Animator a, StructurePanel p) {
	super(a,p);
    }

    public MenuFrame(VisualAnimator va, Animator a) {
	super(va,a);
    }
      
    public MenuFrame(VisualAnimator va, Animator a, StructurePanel p) {
	super(va,a,p);
    }

    public Frame getNewFrame(StructurePanel p) {
	return new MenuFrame(p);
    }

    public Frame getNewFrame() {
	return new MenuFrame();
    }

    public void open() {
	FileDialog fd = new FileDialog(this,
				       "Select file",
				       FileDialog.LOAD);
	fd.show();
	String fileName = fd.getDirectory() + fd.getFile();
	if (fileName == null)
	    return;
	//This if clause is here to avoid several error messages from
	// structureFromFile(filename)
	if(fileName.equals("nullnull"))
	    return;
	try {
	    structureFromFile(fileName);
	} catch (SecurityException e) {
	    Note.show(this, "Applet cannot read local filesystem. " +
		      "See http://www.cs.hut.fi/Research/Matrix/applet/security.html." +
		      "Original exception was:" + e.getMessage());
	}
    }

    protected void structureFromFile(String fileName) {
	int type = FileUtil.sniffType(fileName);
	MatrixFrame mf = null;
	switch(type) {
	case FileUtil.FDT_CLASS:
	    //mf = new MatrixFrame();
	    mf = this;
	    mf.getApplication().addStructure(FileUtil.instantiateFDTClass(fileName));
	    mf.show();
	    break;
	case FileUtil.GRAPH_DESCRIPTION:
	    //mf = new MatrixFrame();
	    mf = this;
	    //mf.getApplication().addStructure(FileUtil.loadGraphFromFile(fileName));
	    FileUtil.loadVisualStructuresFromFile(fileName, mf);
	    mf.show();
	    break;
	case FileUtil.SERIALIZED_ANIMATION:
	    MatrixSerialization ms = FileUtil.openAnimation(fileName);
	    mf = new MatrixFrame(ms.getStructures(), ms.getAnimator());
	    mf.getAnimator().rewind();
	    mf.getStructurePanel().validate();
	    mf.validate();
	    mf.show();
	    break;
	case FileUtil.TABLE_DESCRIPTION:
	    //mf = new MatrixFrame();
	    mf = this;
	    //mf.getApplication().addStructure(FileUtil.loadTableFromFile(fileName));
	    FileUtil.loadVisualStructuresFromFile(fileName, mf);
	    mf.show();
	    break;
	case FileUtil.STRUCTURES_DESCRIPTION:
	    mf = this;
	    //FDT[] temp = FileUtil.loadStructuresFromFile(fileName);
	    //mf.addStructures(temp);
	    FileUtil.loadVisualStructuresFromFile(fileName, mf);
	    mf.show();
	}
    }


    public void save() {
	boolean ok = true;
	File saveFile;
	do {
	    FileDialog fd = new FileDialog(this,
					   "Save As...",
					   FileDialog.SAVE);
	    fd.show();
	    String fileName = fd.getDirectory() + fd.getFile();
	    if (fileName == null)
		return;
	    if(fileName.equals("nullnull"))
		return;
	    saveFile = new File(fileName);
	    if (saveFile.exists()) {
		YesNoDialog yesNo = new YesNoDialog(this, "File " + fileName + " exists! Overwrite?", "File exists");
		ok = yesNo.getResult();
		//		Note.show(this, "cannot overwrite");
		//		ok = false;
	    } else {
		ok = true;
	    }
	} while(!ok);
	getAnimator().end();
	getAnimator().rewind();
	getStructurePanel().validate();
	validate();
	try {
	    MatrixSerialization.save(getAnimator(),
				     getApplication().getStructures(),
				     saveFile);
	} catch (SecurityException e) {
	    Note.show(this, "Applet cannot write local filesystem. " +
		      "See http://www.cs.hut.fi/Research/Matrix/applet/security.html." +
		      "Original exception was:" + e.getMessage());
	    
	}
    }


    public void saveToASCII() {
	boolean ok = true;
	File saveFile;
	do {
	    FileDialog fd = new FileDialog(this,
					   "Save As...",
					   FileDialog.SAVE);
	    fd.show();
	    String fileName = fd.getDirectory() + fd.getFile();
	    if (fileName == null)
		return;
	    if(fileName.equals("nullnull"))
		return;
	    saveFile = new File(fileName);
	    if (saveFile.exists()) {
		YesNoDialog yesNo = new YesNoDialog(this, "File " + fileName + " exists! Overwrite?", "File exists");
		ok = yesNo.getResult();
		//		Note.show(this, "cannot overwrite");
		//		ok = false;
	    } else {
		ok = true;
	    }
	} while(!ok);
	//MatrixFrame f = getFrame();
	MatrixFrame f = this;
	StructurePanel sp = f.getStructurePanel();
	Component[] cs = sp.getComponents();
	ASCIISaver as = new ASCIISaver(cs, saveFile);
	as.saveStructuresToFile();
    }
    

    public void clear() {
	StructurePanel s = getStructurePanel();
	s.removeComponents();
	getAnimator().end();
	getAnimator().disposeUndo();
	s.validate();
	s.repaint();
	validate();
	repaint();
    }
  
    public void export(String format) {
	Panel p = getStructurePanel();
	MatrixFrame f = this;
	MatrixGraphics matrix = new MatrixGraphics(new Rectangle(0, 0, 
								 DEFAULT_WIDTH, DEFAULT_HEIGHT), p.getGraphics());
	boolean ok = true;
	File saveFile;
	do {
	    FileDialog fd = new FileDialog(f, "Export...", FileDialog.SAVE);
	    fd.show();
	    String fileName = fd.getDirectory() + fd.getFile();
	    if (fileName == null) 
		return;
	    if(fileName.equals("nullnull"))
		return;
	    saveFile = new File(fileName);
	    if (saveFile.exists()) {
		Note.show(this, "cannot overwrite");
		ok = false;
	    } else {
		ok = true;
	    }
	} while(!ok);
	
	
	try {
	    if (format.equals("tex")) {      // latex
		p.paint(matrix);
		LatexWriter tolatex = new LatexWriter(matrix.getContents());
		tolatex.write(saveFile);
		return;
	    } else if (format.equals("svg") || format.equals("svgz")) {
		Animator animator = f.getAnimator();
		animator.rewind();
		p.validate();
		f.validate();
		p.paint(matrix);
		boolean svg = format.equals("svg");
		int steps = 0;
		matrix.nextStep();
		while (animator.hasNextOperation()) {
		    animator.redo();
		    p.validate();
		    f.validate();
		    p.paint(matrix);
		    matrix.nextStep();
		    steps++;
		} 
		SVGWriter toSVG = new SVGWriter(matrix.getContents());
		toSVG.setMaxStep(steps);
		toSVG.setBackground(p.getBackground());
		matrix.setScale(1.3);
		if (svg)                        // svg
		    toSVG.write(saveFile);
		else {                          // gzipped svg
		    try {
			File tempFile = new File(saveFile + ".tmp");
			toSVG.write(tempFile);
			FileInputStream fis =
			    new FileInputStream(tempFile);
			GZIPOutputStream zos = new GZIPOutputStream(
								    new FileOutputStream(saveFile));
			int n = -1;
			while( (n=fis.read())!=-1)
			    zos.write(n);
			zos.flush();
			zos.close();
			tempFile.delete();
		    } catch (IOException e) {
			Note.out(e,"Error: " + e.toString()); 
			e.printStackTrace();
		    }
		}	    
	    }
	} catch (SecurityException e) {
	    Note.show(f, "Applet cannot write local filesystem. " +
		      "See http://www.cs.hut.fi/Research/Matrix/applet/security.html." +
		      "Original exception was:" + e.getMessage());
	}
	f.validate();    
	p.validate();
    }

    public void aboutDialog() {
	Dialog d = new AboutDialog(this);
	d.pack();
	d.setVisible(true);
    }


    public Menu createFontMenu() {
	fontMenu = new Menu("Font");
	String defName = getFont().getFamily();
	String[] fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
	for (int j = 0; j < fontNames.length; j++) {
	    //Default font crashes on Linux
  	    if (!fontNames[j].equalsIgnoreCase("default")) {
		CheckboxMenuItem mi = new CheckboxMenuItem(fontNames[j], false);
		if(fontNames[j].equals(defName)) {
		    mi.setState(true);
		}
		// XXX Could we manage with just one itemListener?
		mi.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
			    String comm = e.getItem().toString();
			    int size = getStructurePanel().getFont().getSize();
			    int style = getStructurePanel().getFont().getStyle();
			    getStructurePanel().setFont(new Font(comm,style,size));
			    setActiveFont();
			    invalidateComponents();
			    // are these really required??
			    validateComponents();
			    validate();
			}
		    });
		fontMenu.add(mi);
	    }
	}
	return fontMenu;
    }

    // helper for createFontMenu
    protected void setActiveFont() {
        CheckboxMenuItem mi;
        for (int i = 0; i < fontMenu.getItemCount(); i++) {
            mi = (CheckboxMenuItem)fontMenu.getItem(i);
            mi.setState((mi.getLabel().equals(getStructurePanel().getFont().getName())));
        }
    }

    public void changeFontSize(int size) {
	Font fnt = getFont();
	int style = getFont().getStyle();
	getStructurePanel().setFont(new Font(fnt.getName(),style,size));
	invalidateComponents();
	//XXX are these really necessary?
	validateComponents();
	validate();
    }

    public void newStructure(FDT s) {
	getApplication().addStructure(s);
	validate();
    }

    /**
     * Adds new structure to the StructurePanel.
     *
     * @param s FDT which is going to be added
     * @param name name of the structure
     */
    public void newStructure(FDT s, String name) {
	((StructurePanel)getApplication()).addStructure(s, name);
	validate();
    }

    public void dump() {
	ByteArrayOutputStream buffer = new ByteArrayOutputStream();
	PrintStream out = new PrintStream(buffer);
	getStructurePanel().dump(out);
	getStructurePanel().showMessage("Dump", buffer.toString());
    }

    public void dumpAnimator() {
	ByteArrayOutputStream buffer = new ByteArrayOutputStream();
	PrintStream out = new PrintStream(buffer);
	getAnimator().dump(out);
	getStructurePanel().showMessage("Dump", buffer.toString());

    }



    public void showHistory() {
	state = !state;
	History.getHistory(getStructurePanel()).setVisible(state);
    }

    public void back() {
  	if (getAnimator().hasPreviousOperation()) {
  	    getAnimator().undo();
  	    getStructurePanel().invalidateComponents();
	    validateComponents();
	    validate();
	}
    }

    public void forward() {
  	if (getAnimator().hasNextOperation()) {
  	    getAnimator().redo();
  	    getStructurePanel().invalidateComponents();
	    validateComponents();
	    validate();
	}
	
    }

    public void begin() {
	getAnimator().rewind();
	getStructurePanel().invalidateComponents();
	validateComponents();
	validate();
    }

    public void end() {
	getAnimator().end();
	getStructurePanel().invalidateComponents();
	validateComponents();
	validate();
    }

    public void play() {
	CommandThread commandThread;
	commandThread = new CommandThread(100, new Command() {
		public void execute() {
		    CommandThread ct;
		    while (getAnimator().hasNextOperation()) {
			ct = new CommandThread(500, new Command() {
				public void execute() {
				    getAnimator().redo(); }
			    });
			ct.start();
			try { ct.join(); }
			catch (InterruptedException ex) {}
			getStructurePanel().invalidateComponents();
			validateComponents();
			validate();
		    }
		}
	    });
	commandThread.start();
    }

    public void setBegin() {
	getAnimator().disposeUndo();
	validateComponents();
	validate();
    }

    public void setEnd() {
	getAnimator().disposeRedo();
	validateComponents();
	validate();
    }


}
