/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: SetUp.java,v 1.7 2004/05/25 13:28:26 kstalnac Exp $ */

package matrix.util;

import java.awt.*;
import java.awt.event.*;

public class SetUp extends Frame {
    private String[] chStr = { "Both", "Center", "East", "Horizontal", "None",
    "North", "Norteast", "Nortwest", "Relative", "Remainder", "South", "Southeast",
    "Southwest", "Vertical", "West", "Value" };
    private int[] constants = { GridBagConstraints.BOTH,GridBagConstraints.CENTER,
    GridBagConstraints.EAST,GridBagConstraints.HORIZONTAL,GridBagConstraints.NONE,
    GridBagConstraints.NORTH,GridBagConstraints.NORTHEAST,GridBagConstraints.NORTHWEST,
    GridBagConstraints.RELATIVE,GridBagConstraints.REMAINDER,GridBagConstraints.SOUTH,
    GridBagConstraints.SOUTHEAST,GridBagConstraints.SOUTHWEST,GridBagConstraints.VERTICAL,
    GridBagConstraints.WEST };
    
    private GridBagConstraints gbconst;
    private Container container;
    
    public SetUp(Container container, GridBagConstraints gbc) {
        this.container = container;
        this.gbconst = gbc;
            
        Button set = new Button("Set");
        Button reset = new Button("Reset");
        Button cancel = new Button("Cancel");
        Button dismiss = new Button("Dismiss");
        Panel buttons = new Panel();
        
        Choice gridx = new Choice();
        addItems(gridx, "x", chStr, gbc.gridx);
        Choice gridy = new Choice();
        addItems(gridy, "y", chStr, gbc.gridy);
        
        Choice gridwidth = new Choice();
        addItems(gridwidth, "w", chStr, gbc.gridwidth);
        Choice gridheight = new Choice();
        addItems(gridheight, "h", chStr, gbc.gridheight);
        
        Choice fill = new Choice();
        addItems(fill, "f", chStr, gbc.fill);
        
        Choice ipadx = new Choice();
        addItems(ipadx, "ix", chStr, gbc.ipadx);
        Choice ipady = new Choice();
        addItems(ipady, "iy", chStr, gbc.ipady);
        
        // insets???
        
        Choice anchor = new Choice();
        addItems(anchor, "a", chStr, gbc.anchor);
        
        //Choice weightx = new Choice();
        //addItems(weightx, chStr, gbc.weightx);
        //Choice weighty = new Choice();
        //addItems(weighty, chStr, gbc.weighty);
        
        setLayout(new BorderLayout());

        buttons.add(set, "North");
        buttons.add(reset, "North");
        buttons.add(cancel, "North");
        buttons.add(dismiss, "North");
        add(buttons, "North");
        
        Panel grids = new Panel();
        grids.add(new Label("gridx"));
        grids.add(gridx);
        grids.add(new Label("gridy"));
        grids.add(gridy);
        grids.add(new Label("gridwidth"));
        grids.add(gridwidth);
        grids.add(new Label("gridheight"));
        grids.add(gridheight);
        add(grids, "Center");
        
        Panel anc = new Panel();
        anc.add(new Label("anchor"));
        anc.add(anchor);
        
        Panel fl = new Panel();
        fl.add(new Label("fill"));
        fl.add(fill);
        
        Scrollbar weightx = new Scrollbar(Scrollbar.HORIZONTAL);
        Note.out(this, "value = " + gbconst.weightx);
        weightx.setValue((int)(10*gbconst.weightx));
        weightx.setMinimum(0);
        weightx.setMaximum(100);
        weightx.setUnitIncrement(1);
        weightx.setBlockIncrement(1);
        //weightx.setVisibleAmount(10);
        weightx.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                Note.out(this, "set value = " + e.getValue());
                gbconst.weightx = e.getValue()/100;
            }
        });
        Scrollbar weighty = new Scrollbar(Scrollbar.VERTICAL);
        weighty.setMaximum(100);
        
        Panel ipads = new Panel();
        ipads.add(anc);
        ipads.add(fl);
        ipads.add(new Label("ipadx"));
        ipads.add(ipadx);
        ipads.add(new Label("ipady"));
        ipads.add(ipady);
        ipads.add(new Label("weightx"));
        ipads.add(weightx);
        ipads.add(new Label("weighty"));
        ipads.add(weighty);
        add(ipads, "South");
        add(weightx, "South");
        
    }
    
    private int getDefault(int def) {
        for (int i=0; i<constants.length; i++)
            if (def == constants[i])
                return i;
        return chStr.length;
    }
    
    private void addItems(Choice c, String name, String[] items, int def) {
        for (int i=0; i<items.length; i++)
            c.addItem(items[i]);
        c.select(getDefault(def));
        c.setName(name);
        c.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) { select(e); }
        });
    }
    
    public void select(ItemEvent e) {
        Choice c = (Choice)e.getItemSelectable();
        String s = (String)e.getItem();
        int i = constants[c.getSelectedIndex()];
        String n = c.getName();
        if (n == null) n = "UNKNOWN";
        
        if (n.equals("x")) gbconst.gridx = i;
        else if (n.equals("y")) gbconst.gridy = i;
        else if (n.equals("w")) gbconst.gridwidth = i;
        else if (n.equals("h")) gbconst.gridheight = i;
        else if (n.equals("ix")) gbconst.ipadx = i;
        else if (n.equals("iy")) gbconst.ipady = i;
        else if (n.equals("a")) gbconst.anchor = i;
        else if (n.equals("f")) gbconst.fill = i;
        else Note.show(this, "Unknown selection " + n);
        
        Note.out(this, "set " + n + " for " + container);
        ((GridBagLayout)container.getParent().getLayout()).setConstraints(container, gbconst);
        container.validate();
    }
    
}
