/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: MG_Element.java,v 1.13 2004/05/26 08:06:02 vkaravir Exp $ */

package matrix.util.export;

import java.awt.Color;
import java.io.*;
import java.util.Hashtable;

/**
 * Abstract superclass for all elements used in export.
 */
public abstract class MG_Element {

    /** Stores this elements states at each step */
    protected Hashtable steps;
    /** Opacity of this element */
    private float opacity;
    /** Color of this element */
    protected Color color;
    public static final int MG_ARC = 0;
    public static final int MG_LINE = 1;
    public static final int MG_OVAL = 2;
    public static final int MG_POLYGON = 3;
    public static final int MG_RECT = 4;
    public static final int MG_TEXT = 5;


    /**
     * Constructs new <code>MG_Element</code>.
     */
    public MG_Element() {
	opacity = 1;
	steps = new Hashtable();
	steps.put(new Integer(0), this);
    }

    /**
     * Returns true if this element has the given step, otherwise
     * returns false.
     * @param step the step
     */
    public boolean hasStep(int step) {
	return steps.containsKey(new Integer(step));
    }

    /**
     * Returns the color of this element. 
     * @return the color of this element
     */
    public Color getColor() { return color; }

    /** 
     * Sets the opacity of this element to the given value
     * @param o the new opacity
     */
    public void setOpacity(float o) { opacity = o; }

    /** 
     * Returns the opacity of this element.
     * @return The opacity of this element. 
     */
    public float getOpacity() { return opacity; }

    /**
     * Adds the given element to the given step of this element.
     * @param elem the element to add
     * @param step the step
     */
    public void addStep(MG_Element elem, int step) {
	if (!hasStep(step)) {
	    steps.put(new Integer(step), elem);
	}
    }

    /** Returns the element at the specified step */
    public abstract MG_Element getStep(int step);
    /** Returns LaTeX representation of this element */
    public abstract String toLatex();

    /**
     * Writes SVG representation of this element to the given
     * <code>Writer</code>.
     * @param out the writer for writing SVG
     * @param stepLength length of one step of animation (seconds)
     * @param pause lause between steps (seconds)
     */
    public abstract void toSVG(Writer out, SVGUtil util)
	throws IOException;

    /** Returns the type of this element */
    public abstract int getType();
}
