/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.Dimension;

import matrix.structures.FDT.FDT;
import matrix.structures.code.*;

/**
 * This is the container for single visual code line representation.
 *
 * @author Ari Korhonen
 * @version $Revision: 1.8 $
 */
public class VisualCodeLine extends VisualNode {
    CodeLine codeline;
    Code code;

    public VisualCodeLine(FDT fdt) {
        super(fdt);
    }

    public VisualCodeLine(Code code, CodeLine codeline) {
        this(new StyledCodeLine(code, codeline));
        this.code = code;
        setLine(codeline);
        //super(code);
        setName("CodeLine");
    }

    /** VisualCodeLine is conserned to be visited if its time stamp equals global time. 
     * @return true if this line is currently active (to be executed), false otherwise
     */
    public boolean isVisited() {
        if (code.getTime() == codeline.getStamp())
            return true;
        else
            return false;
    }

    public void setLine(CodeLine cl) {
        this.codeline = cl;
    }
    
    /** This class overrides this method return false. 
     * @see VisualNode#isRoundRectangle()
     */
    public boolean isRoundRectangle() { return false; }

    static int commonWidth = 0;
    public Dimension getPreferredSize() {
        Dimension d = super.getPreferredSize();
        if (d.width > commonWidth)
            commonWidth = d.width;
        else
            d.width = commonWidth;
        return d;
    }
}


