/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.*;

import matrix.decoration.*;
import matrix.structures.CDT.CDT;
import matrix.structures.FDT.FDT;
import matrix.util.Note;

/** VisualHotSpot is an abstraction for special active components within containers.
 * @see VisualContainer
 *
 * @author Ari Korhonen
 * @version $Revision: 1.9 $
 */
public class VisualHotSpot extends VisualType {
    //private int x,y;
    private String label;


    // --- constructors ---

    public VisualHotSpot(FDT fdt) {
        super(fdt);
        label = "hotspot";
    }

    public String getLabel() { return label; }

    public void setLabel(String label) { this.label = label; }

    public void assign() {
        Note.show(this, this+": Action (assing) not defined for this HotSpot");
    }

    /** Returns the preferred size of this component.
     * By default the preferred size of this component
     * consists of the insets and the size of the label string
     */
    public Dimension getPreferredSize() {
        Font f = getFont();
        int w = 1;
        int h = 1;
        if (f != null) {
            w = getFontMetrics(f).stringWidth(label);
            h = getFontMetrics(f).getHeight();
        }
        return new Dimension(w, h);
    }

    /** This method draws the representation of this hot spot
     */
    public void paint(Graphics g) {
        Rectangle r = getBounds();
        //Note.out(this, this + ": r = " + r);
        int nWidth = getFontMetrics(getFont()).stringWidth(label);
        int nHeight = getFontMetrics(getFont()).getHeight();
        int descent = getFontMetrics(getFont()).getDescent();
        setBackgroundColor(g);
        g.fillRect(r.x,r.y,r.width,r.height);
        setPenColor(g);
        g.drawString(label,
                     r.width/2 - nWidth/2,
                     nHeight - descent);
        //g.drawString("xxx", 1,nHeight-descent);
        super.paint(g);
    }

    //this overrides the method in VisualType
    public void setBackgroundColor(Graphics g) {
  	Color defaultColor;
        ColorDecorator cd;
	if (fdt instanceof ColorDecorator) {
            cd = (ColorDecorator)fdt;
	} else if (fdt instanceof Styled) {
            cd = ((Styled)fdt).getStyleSheet();
	} else 
            cd = getDefaultStyleSheet();
        if (isVisited()) {
            if (hasFocus()) 
                defaultColor = cd.getVisitedFocusedBackgroundColor();
            else
                defaultColor = cd.getVisitedBackgroundColor();
        } else {
            if (hasFocus()) {
                defaultColor = cd.getFocusedBackgroundColor();
		//if the structure is CDT, then focused background color for 
		//the hotspot is fetched from VisualContainer
		if(((VisualType)getParent()).getStructure() instanceof CDT)
		    defaultColor = ((VisualContainer)getParent()).getCDTColor();
	    }            
	    else
                defaultColor = cd.getDefaultBackgroundColor();
        }
        if (g != null) {
	  //Note.out(this, "Setting backgroundcolor for " + this + " to " + defaultColor +"\n" +
	  //	   "element " + (isVisited() ? "is visited and " : "") + 
	  //(hasFocus() ? "has focus" : "" ));
            g.setColor(defaultColor);
	}
    }

}
