/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualKey.java,v 1.46 2004/06/17 15:28:55 archie Exp $ */

package matrix.visual;

import java.awt.*;

import matrix.decoration.*;
import matrix.structures.EDT.*;
import matrix.structures.FDT.*;
import matrix.structures.memory.Key;
import matrix.util.Note;

public class VisualKey extends VisualPrimitive {
    protected Key key;
    private java.awt.Component awtComponent = null;

    public static int MIN_SIZE = 8;
    
    public VisualKey(Key k) {
        super(k);
	setName(k.toString());
        key = k;
    }
    /*    
    public int getSemantics() { 
	return VisualContainer.getSemantics(); 
    }
    */
    public void rename() {
	if (getParent() instanceof VisualReference) {
	    ((VisualReference)getParent()).rename();
	    setInvalid();
	    return;
	}
        rename(getApplication().getInput("Rename: ", getName()));
    }

    public void rename(String newName) {
        if (newName.equals(getName()))
            return;
	getAnimator().startOperation();
        key = new Key(newName);
        setName(getKey());
        read = this;
        write();
        setInvalid();
	getAnimator().endOperation();
    }

    public void doLayout() {
        Font font;
        if ((isVisible()) && ((font = getFont()) != null)) {
            if (isEmpty()) {
                width = 4;
                height = 4;
            } else {
                if (getKey().length() == 1)
                    width = getFontMetrics(font).stringWidth("99");
                else
                    width = getFontMetrics(font).stringWidth(getKey());
                height = getFontMetrics(font).getHeight();
            }
        }
    }
    
    public Dimension getPreferredSize() { 
	if(width < MIN_SIZE && height < MIN_SIZE) {
	  /*
	    if(getParent() instanceof VisualArrayComponent) {
		Component c1 = getParent();
		Component c2 = null;
		while(c1 instanceof VisualType) {
		    c2 = c1;
		    c1 = c1.getParent();
		}
		if(c2 instanceof VisualArray)
		    return new Dimension(MIN_SIZE * 2, MIN_SIZE * 2);
	    } else
	  */
            return new Dimension(MIN_SIZE, MIN_SIZE);
	}
	if(width < MIN_SIZE) 
	    return new Dimension(MIN_SIZE, height);
	if(height < MIN_SIZE) 
	    return new Dimension(width, MIN_SIZE);
        return new Dimension(width, height);
    }
    
    public String getKey() {
	// if this is label of a reference (weight) then ask key from the underlying structure
	if (getParent() instanceof VisualReference && !(((VisualReference)getParent()).getTarget() instanceof VisualArrow)) {
	    VisualComponent vc = ((VisualReference)getParent()).getSource();
	    if (vc.getStructure() instanceof LabelDecorator) {
		LabelDecorator ld = (LabelDecorator)vc.getStructure();
		key.setKey(new Key(ld.getReferenceLabel(vc.getReferenceNumber(((VisualReference)getParent()).getTarget()))));
	    }
	}
        return key != null ? 
            ((key.toString() != null) ? key.toString() : "") : 
            "<E>";
    }

    public void setKey(String s) {
        if (getKey() != s)
            key.setKey(new Key(s));;
    }

    /* We cannot assign anything to keys, thus we call parent's write method. */
    public void assign() {
        if (getParent() instanceof VisualType) {
	    ((VisualType)getParent()).write();
	    /*
	    // If the FDT this key's parent is representing 
            // is a SimulationTree2, the read
	    // structure is inserted to it. 
            // Otherwise, we assign into the parent.
	    // Should this be done?
	    FDT f;
	    if((f = ((VisualType)getParent()).getStructure()) instanceof SimulationTree2) 
		f.setElement(read.getStructure());
	    else {
		((VisualType)getParent()).assign();
	    }
	    */
	}
    }
    
    public void selectEntity(boolean isShiftDown) {
	key.select();
    }
    
    public void delete() {
        if (getParent() instanceof VisualType)
            ((VisualType)getParent()).removeKey(key);
    }
    
    public void write() {
        ((VisualType)getParent()).write();
    }
    
    private final static int ROUND = 15;
    
    public FDT getStructure() {
	// new needed because key is a primitive type
	/* Huh? Could someone elaborate on this comment? */
	if (key instanceof Visitable) {
	    Key k = new VisitableKey(key);
	    k.setOrig(key);
	    return k;
	} else if (key instanceof ColorableKey) {
	    Key k = new ColorableKey(key);
	    k.setOrig(key);
	    return k;
	} else {	    
	    Key k = new Key(key);
	    k.setOrig(key);
	    return k;
	}
    }

    /**
     * Determines whether this component has been visited by looking first at the FDT and if it is not
     * Visitable then by returning isVisited flag. 
     * @return true if this component has been visited; false otherwise.
     */
    public boolean isVisited() {
	if ((key != null) && (key instanceof Visitable)) {
	    return ((Visitable)key).isVisited();
	}
	return super.isVisited() || key.isSelected(); 
	
    }
    /*
    public boolean isVisited() {
	return (super.isVisited() || key.isSelected());
    }
    */
    public boolean isEmpty() {
        return getKey().trim().equals(Key.EMPTY);
    }
    private Font lastFont = null;
    private String lastKey = null;
    private int textX = 0, textY = 0;
    public void paint(Graphics g) {
        super.paint(g);
        if (g == null) Note.warning(this, "Graphics not defined");
        int x = 1;
        int y = 1;
        int w = width-2;
        int h = height-2;

	setBackgroundColor(g);
        g.fillRect(x, y, w, h);

        Font font = getFont();
        String key = getKey();
        if (font != null) {
            if ((font != lastFont) || (key != lastKey)) {
                FontMetrics fm = getFontMetrics(font);
                lastFont = font;
                textX = x + (w - fm.stringWidth(key))/2;
                textY = y + h - (h - fm.getHeight())/2 - fm.getDescent();
            }
            //setColor(g);
	    setPenColor(g);
            g.drawString(key, textX, textY);
            //resetColor(g);
        }
    }

    public static void fontSizeChanged(int newFontSize) {
	MIN_SIZE = newFontSize/2;
	if (MIN_SIZE == 0)
	    MIN_SIZE = 1;
    }
}
