/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: VisualPrimitive.java,v 1.12 2004/05/26 08:06:11 vkaravir Exp $ */

package matrix.visual;

import java.awt.Color;
import java.io.PrintStream;

import matrix.decoration.*;
import matrix.structures.FDT.FDT;

public abstract class VisualPrimitive extends VisualType {

    /* Current color of this VisualType */
    protected Color color;

    /** Debug method. 
     * Subclasses should invoke super.debug() in order
     * to provide all debug information. Every subclass has its own
     * responsibility to provide debug info about its instance variables.
     * The debug info should be printed out with Note.out() utility.
     */
    public void dump(PrintStream out) {
        super.dump(out);
        boolean ISCEFlag;
        ISCEFlag = ((VisualType)getParent()).isSubComponentsEnabled();
        out.println("[[parent.ISCE="+ISCEFlag+
                 "; parent = " + getParent() +
                 "; color="+color+"]]");
    }

    /** Construct the representation for the specified fundamental 
     * primitive data type. 
     */
    public VisualPrimitive(FDT t) {
        super(t);
    }

    /** do nothing, since primitives cannot be shown independently */
    public void newWindow() {}

    /** do nothing, since primitives cannot have new representation */
    public void newRepresentation() { }



    // -- visualization methods: COLOR --

    private StyleSheet defaultStyleSheet = null;
    public StyleSheet getDefaultStyleSheet() {
	if (defaultStyleSheet == null) {
            defaultStyleSheet = new StyleSheetAdapter();
	    defaultStyleSheet.setFocusedBackgroundColor(Color.red);
	}
        return defaultStyleSheet;
    }
}
