/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.visual;

import java.awt.*;

import matrix.structures.code.Code;

/**
 * This is a container for text
 *
 * @author Ari Korhonen
 * @version $Revision: 1.10 $
 */
public class VisualTextPanel extends VisualContainer {
    Code code;

    public VisualTextPanel(Code code) {
        this.code = code;
        setName("Variables");
    }

    public void setCode(Code code) { this.code = code; }

    public boolean canInsert() {
	return false;
    }

    public void insert(Object o) {}

    public void validateComponents() {
        Component[] cmps = getItems();
        String[] text = code.getVariables();
        String[] desc = new String[text.length];
        String[] d = code.getVariableDescription();
        for (int i=0; i<text.length; i++) 
            desc[i] = (d.length > i) ? d[i] : "no description??";
        
        for (int i=0; i<text.length; i++) {
            if (i >= cmps.length) // missing visual component
                add(new VisualTextLine(text[i]), desc[i]);
            else ((VisualTextLine)cmps[i]).setLine(text[i], desc[i]);
        }
        while (cmps.length > text.length) removeItem(text.length);
    }
    
    public LayoutSize doActualLayout() {
        Component[] cmps = getItems();
        int newWidth = 0;
        int newHeight = 0;
        int x = 0;
        int y = 0;
        for (int i=0; i<cmps.length; i++) {
            VisualComponent vcl = (VisualComponent)cmps[i];
            LayoutSize vclSize = vcl.lGetPreferredSize();
            if (vclSize.width > newWidth) newWidth = vclSize.width;
            newHeight += vclSize.height;
            vcl.lSetBounds(x,y,vclSize.width,vclSize.height);
            y += vclSize.height;
        }
        return new LayoutSize(newWidth, newHeight);
    }

    public void paint(Graphics g) {
        super.paint(g);
        if (isPicked()) return; // no frames
        paintFrame(g);
    }
}


