/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package prototype.ui;

import java.awt.*;

import content.exercises.*;

import matrix.animation.Animator;
import matrix.uitools.*;
import matrix.util.Note;

public class ExerciseFrame extends MenuFrame {

  private SimulationExercise exercise;
  private ModelAnswerFrame maf = null;

  Button gradeButton;
 
  TextArea descriptionTextArea ;

  public ExerciseFrame(Animator a, SimulationExercise exercise) {
    super(a, new ExercisePanel(a, exercise));
    setTitle("Exercise");
    this.exercise = exercise;
    descriptionTextArea.setText(exercise.getDescription());
    getVisualAnimator().setResetButtonVisible(false);
    getVisualAnimator().setPlayButtonVisible(false);
    setSize(640, 640);
  }

  public ExerciseFrame(SimulationExercise exercise) {
    this(new Animator(), exercise);
  }


  protected void layoutComponents() {

    descriptionTextArea = new TextArea("");

    // See java.awt.GridBagLayout for details.
    GridBagLayout grid = new  GridBagLayout();
    setLayout(grid); 
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.insets = new Insets(10,10,10,10);
    gbc.gridy = 0;
    gbc.weightx = 0.0;
    gbc.weighty = 0.0;
    gbc.fill = GridBagConstraints.BOTH;
    add(descriptionTextArea, gbc);

    gbc.gridy = 1;
    gbc.weightx = 0.0;
    gbc.weighty = 0.0;
    gbc.fill = GridBagConstraints.NONE;
    add(getVisualAnimator(), gbc);
    
    gbc.gridy = 2;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    ScrollPane sp = getScrollPane();
    sp.add(getStructurePanel());
    add(sp, gbc);
    
  }



 protected MenuBar createMenuBar() {
     return MenuFactory.getInstance().getMenuBar(this);
  }




  ExercisePanel getExercisePanel() {
    return (ExercisePanel)getStructurePanel();
  }
  
  

  public void reset() {
      //bugi, bugi exercise was already defined, Ville 8.4.03
      //SimulationExerciseModel exercise = (SimulationExerciseModel)getExercisePanel().getExercise();
      if (maf != null) {
	  maf.close();
	  maf = null;
      }
      try {
	  exercise = (SimulationExercise)exercise.getClass().newInstance();
      } catch (Exception e) {
	  Note.show(this, "ERROR: " + e.getMessage());
	  Note.err(this, e.getMessage());
	  Note.printStackTrace();
      }
      long seed = System.currentTimeMillis();
      ((SimulationExercise)exercise).setSeed(seed);
      getExercisePanel().setExercise(exercise);
      getExercisePanel().reset();
      
      descriptionTextArea.setText(exercise.getDescription());
      validate();
      getExercisePanel().repaint();
  }

 
  public void modelAnswer() {
    if(exercise instanceof SimulationExerciseModel) {
      maf = new ModelAnswerFrame((SimulationExerciseModel)exercise);
      maf.show();
    } else {
      Note.show(this, "Cannot show model answer, none given.");
    }
  }


  public void grade() {
    if(exercise instanceof SimulationExerciseModel) {
      Judgement j = Jury.judge(((SimulationExercise)exercise).getAnswer(), getAnimator(), (SimulationExerciseModel)exercise);
      new GradeDialog(j).show();
    } else if(exercise instanceof MultipleChoiceExercise) {
      MultipleChoiceExercise mce = (MultipleChoiceExercise) exercise;
    } else {
      Note.show(this, "Cannot judge this exercise. No method of judging given.");
    }
  }
  


  

}
