/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package prototype.ui;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;

import content.exercises.*;

import matrix.animation.Animator;
import matrix.simulation.VisualTypeConf;
import matrix.uitools.StructurePanel;
import matrix.visual.*;

public class ExercisePanel extends StructurePanel {

    public static final boolean DEBUG = true;

    SimulationExercise exercise;

    private boolean validated = false;

    private boolean swapEnabled = false;

    // constructors


    /**
     */
    public ExercisePanel(Animator animator, SimulationExercise e) {
        super(animator);
        this.exercise = e;
        initExercise(e);
    }
    
	

    public void setExercise(SimulationExercise e) {
	exercise = e;
    }

    public SimulationExercise getExercise() {
	return exercise;
    }
    

    protected void initExercise(SimulationExercise e) {

        // animator is changed in case the active animator is some
        // other animator (than the animator of this frame)
      Animator.setActiveAnimator(getAnimator());

      VisualType[] vt = null;
      // This if-else clause will create the data structures used in the exercise.
      if(e instanceof StyledExercise) {
        vt = addStructures(e.init(), ((StyledExercise)e).getStructureVisualisations());
      } else {
        vt = addStructures(e.init());
      }

      //getAnimator().endOperation();
      //getAnimator().disposeUndo();

      String[] names = e.getStructureNames();
      for (int i = 0; i < names.length; i++) {
        vt[i].setName(names[i]);
      }

      // rotates containers when needed
      if(e instanceof HasRotatedContainers){
	  boolean[] rotateInfo = ((HasRotatedContainers)e).getContainerRotations();
	  for (int i = 0; i < vt.length; i++) {
	      if (vt[i] instanceof VisualContainer)
		  ((VisualContainer)vt[i]).setRotated(rotateInfo[i]);
	  }	  
      }


      // disable hotSpot
      for (int i = 0; i < vt.length; i++) {
          if (vt[i] instanceof VisualContainer) { // should be
              ((VisualContainer)vt[i]).setHotSpotEnabled(false);
          }
      }


        if (e instanceof ConfigureVisualType) {
            VisualTypeConf[] conf = ((ConfigureVisualType)e).conf();
            for (int i = 0; i < vt.length; i++) {
                vt[i].configure(conf[i]);
            }
        }

        if (e instanceof ToggleSwapButton) {
            addToggleSwapButton();
        }

	
        if (e instanceof ButtonExercise) {
	    Button[] buttons = makeButtons((ButtonExercise)e);
	    layoutButtons(e, buttons);
	}

	if (e instanceof AWTComponentUtilizer) {
	    // buttons are defined in the exercise class
	    Component[] components = ((AWTComponentUtilizer)e).getAWTComponents();
	    for (int i = 0; i < components.length; i++) {
		add(components[i]);
	    }
	    ((AWTComponentUtilizer)e).setApplication(this);
	}
	
    }


    
    
    protected void layoutButtons(Exercise e, Button[] buttons) {
	    
	if (e instanceof ButtonGroups) {
	    // top and botton insets=5, left and right insets = 5
	    Insets buttonInsets = new Insets(2,20,2,20);
	    int[] groups = ((ButtonGroups)e).buttonGroups();
	    int k = 0;
	    for (int i = 0; i < groups.length; i++) {
		Panel panel = new Panel();
		while (k < groups[i]) {
		    panel.add(buttons[k++]);
		}
		addComponent(panel, buttonInsets);
		k = groups[i];
	    }
	    Panel panel = new Panel();
	    for (int i = k; i < buttons.length; i++) {
		panel.add(buttons[i]);
	    }
	    addComponent(panel,buttonInsets);
	}
	
	else {
	    for (int i = 0; i < buttons.length; i++) {
		addComponent(buttons[i]);
	    }
	}
    }



    protected Button[] makeButtons(ButtonExercise bex) {
        final String[] buttons = bex.buttonNames();
        final String[] commands = bex.buttonCommands();
        Button[] but = new Button[buttons.length];
        for (int i = 0; i < buttons.length; i++) {
            final Button b = new Button(buttons[i]);
	    
            b.setActionCommand(commands[i]);
            b.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent evt) {
                        reflection(b.getActionCommand());
                    }
                });
            but[i] = b;
        }
        return but;
    }
    

    
    public void reflection(String cmd) {
        String param = null;
        int p1 = cmd.indexOf('(');
        int p2 = cmd.lastIndexOf(')');
        if (p1 < p2) {
            param = cmd.substring(p1+1, p2);
            cmd = cmd.substring(0, p1);
        }
        Class cl = getClass();
        do {
            Method[] m = cl.getDeclaredMethods();
            for (int i=0; i<m.length; i++) {
                if (m[i].getName().equals(cmd)) {
                    Class[] c = m[i].getParameterTypes();
                    if ((c.length == 0) && (param == null)) {
                        // methods with no parameters
                        try { m[i].invoke(this, null); }
                        catch (Exception e) {
                            if(e instanceof InvocationTargetException) {
                                Throwable invoked = ((InvocationTargetException)e).getTargetException();
                                invoked.printStackTrace();
                                showMessage("InvocationTargeException", invoked.toString());
                            } else {
                                e.printStackTrace();
                                showMessage("Exception", e.toString());
                            }
                        }
                        return;
                    } else if ((c.length == 1) && (param != null)) {
                        // methods with single parameter
                        Object[] parameter = new Object[] { param };
                        try { m[i].invoke(this, parameter); }
                        catch (Exception e) {
                            if(e instanceof InvocationTargetException) {
                                Throwable invoked = ((InvocationTargetException)e).getTargetException();
                                invoked.printStackTrace();
                                showMessage("InvocationTargetException", invoked.toString());
                            } else {
                                e.printStackTrace();
                                showMessage("Exception", e.toString());
                            }
                        }
                        return;
                    }
                }
            }
            cl = cl.getSuperclass();
        } while (cl != null);
        showMessage("Error", "Method " + cmd + " not found in " + getClass());
    }


    public void reflectSelectedVisualType(String cmd) {
        if (VisualType.read != null) {
            VisualType.read.reflection(cmd);
        }
    }


    public void reset() {
        getAnimator().rewind();
	getAnimator().disposeRedo();
        removeComponents();
	removeAll(); // Otherwise AWT-components are not removed

	repaint();
        initExercise(exercise);
	addEmptyComponentToTheBottom();
	validate(); //ADDED 3.2.2004
	            //(otherwise panels have incorrect lengths after reset)
    }


    public void setSwapEnabled(boolean value) {
	swapEnabled = value;
	if (swapEnabled) {
	    turnSwapOn();
	} else {
	    turnSwapOff();
	}
    }


    
    public boolean getSwapEnabled() {
	return swapEnabled;
    }
    

    private void turnSwapOn() {
        VisualType.setSemantics(VisualType.SWAP_SEMANTICS);
    }


    private void turnSwapOff() {
        VisualType.setSemantics(VisualType.INSERT_SEMANTICS);
    }



    public void addToggleSwapButton() {
        final Panel panel = new Panel();
        final Label label = new SwapLabel();
        final Button button = new Button("Toggle copy mode / swap mode");
        button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    if (label.getText().equals("copy mode")) {
                        label.setText("swap mode");
                        turnSwapOn();
                        validate();
                    } else {
                        label.setText("copy mode");
                        turnSwapOff();
                        validate();
                    }
                }
            });
        panel.add(button);
        panel.add(label);
        addComponent(panel);
    }


    


}


