/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2004 Matrix group
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package prototype.ui;

import java.awt.MenuBar;

import content.exercises.*;

import matrix.animation.Animator;
import matrix.structures.FDT.FDT;
import matrix.uitools.*;
import matrix.visual.VisualType;

public class ModelAnswerFrame extends MenuFrame {

  public ModelAnswerFrame(SimulationExerciseModel mod) {
    super(new Animator());
    setTitle("Model answer");
    getVisualAnimator().setResetButtonVisible(false);
    getVisualAnimator().setPlayButtonVisible(false);
    StructurePanel p = getStructurePanel();

    Animator.setActiveAnimator(getAnimator());
    FDT[] answer = mod.solve();

    VisualType[] vt = null;
    if(mod instanceof StyledExercise) {
      vt = p.addStructures(answer, ((StyledExercise)mod).getModelAnswerVisualisations());
    } else {
      vt = p.addStructures(answer);
    }

    if (mod instanceof ModelAnswerNames) {
	String[] names = ((ModelAnswerNames)mod).getModelAnswerNames();
	for (int i = 0; i < vt.length; i++) {
	    vt[i].setName(names[i]);
	}
    }

    //Rotates containers if requested
    if(mod instanceof HasRotatedContainers){
	boolean[] rotateInfo = ((HasRotatedContainers)mod).getContainerRotations();
	for (int i = 0; i < vt.length; i++) {
	    if (vt[i] instanceof matrix.visual.VisualContainer)
		((matrix.visual.VisualContainer)vt[i]).setRotated(rotateInfo[i]);
	}	  
    }

    getStructurePanel().disableVisualComponents();
    getVisualAnimator().validate();
  }


  protected MenuBar createMenuBar() {
      return MenuFactory.getInstance().getMenuBar(this);
  }


}
