/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2001  Ari Korhonen and others
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import matrix.structures.memory.VirtualArray;
import matrix.structures.FDT.FDT;
import matrix.structures.FDT.LinkedList;

/**
 * A subclass of VisualizableList that can be visualized
 * and animated using Matrix. Since instance variables
 * are now stoed inside a VirtualArray instead of 
 * normal instance fields quite a few methods need to
 * be overridden.
 */

public class AnimatedList extends VisualizableList {

  // contents[0] holds the key and 
  //contents[1] link to next element
  private VirtualArray contents;

  public AnimatedList() {
    this(null);
  }

  public AnimatedList(Object contents) {
    this.contents = new VirtualArray();
    this.contents.setObject(contents, 0);
  }

  // Overridden from PlainList
  public Object getContents() {
    return contents.getObject(0);
  }

  // Overridden from PlainList
  public void setContents(Object o) {
    contents.setObject(o, 0);
  }

  // Overridden from PlainList
  public PlainList getNextElement() {
    return (PlainList) contents.getObject(1);
  }

  // Overridden from PlainList
  public void setNextElement(PlainList p) {
    contents.setObject(p, 1);
  }

  // Overridden from VisualizableList
  public LinkedList getNewNode(Object key) {
    return new AnimatedList(key);
  }

}
