/*
 * Matrix -- algorithm simulation system
 * Copyright (C) 2000-2001  Ari Korhonen and others
 * (See the AUTHORS file and the author tag below for more information
 * on the authors.)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import matrix.structures.FDT.FDT;
import matrix.structures.FDT.LinkedList;

/**
 * A subclass of PlainList that can be visualized in
 * Matrix, but cannot be used for algorithm animation.
 * 
 * This implementation of linked list can be visualized
 * as a linked list in Matrix.
 */
public class VisualizableList extends PlainList implements LinkedList {

  public VisualizableList() {
    super();
  }

  public VisualizableList(Object contents) {
    super(contents);
  }

  //--- interface FDT ---

  public Object getElement() {
    return getContents();
  }

  public void setElement(Object o) {
    setContents(o);
  }

  //--- interface LinkedList ---

  public LinkedList getNext() {
    return (VisualizableList) getNextElement();
  }

  public void setNext(LinkedList l) {
    // to avoid ClassCastException
    if(l instanceof VisualizableList) {
      setNextElement(l);
    }
  }

  public boolean hasNext() {
    return getNextElement != null;
  }

  public LinkedList getNewNode(Object key) {
    return new VisualizableList(key);
  }
  
}
